# Formatting configuration for locale de_CH

languages={'gv': 'Manx', 'gu': 'Gujarati', 'gd': u'Schottisch-G\xe4lisch', 'ga': 'Irisch', 'gn': 'Guarani', 'gl': 'Galizisch', 'la': 'Latein', 'ln': 'Lingala', 'lo': 'Laotisch', 'tt': 'Tatarisch', 'tr': u'T\xfcrkisch', 'ts': 'Tsonga', 'lv': 'Lettisch', 'to': 'Tongaisch', 'lt': 'Litauisch', 'tk': 'Turkmenisch', 'th': 'Thai', 'ti': 'Tigrinja', 'tg': 'Tadschikisch', 'te': 'Telugu', 'haw': 'Hawaiianisch', 'yi': 'Jiddisch', 'yo': 'Joruba', 'de': 'Deutsch', 'da': u'D\xe4nisch', 'dz': 'Bhutanisch', 'st': 'Sotho', 'qu': 'Quechua', 'el': 'Griechisch', 'eo': 'Esperanto', 'en': 'Englisch', 'zh': 'Chinesisch', 'za': 'Zhuang', 'uk': 'Ukrainisch', 'eu': 'Baskisch', 'et': 'Estnisch', 'es': 'Spanisch', 'ru': 'Russisch', 'rw': 'Ruandisch', 'rm': u'R\xe4toromanisch', 'rn': 'Kirundi', 'ro': u'Rum\xe4nisch', 'bn': 'Bengalisch', 'be': u'Wei\xdfrussisch', 'bg': 'Bulgarisch', 'ba': 'Baschkirisch', 'wo': 'Wolof', 'jv': 'Javanisch', 'bo': 'Tibetisch', 'bh': 'Biharisch', 'bi': 'Bislama', 'br': 'Bretonisch', 'ja': 'Japanisch', 'om': 'Oromo', 'oc': 'Okzitanisch', 'tw': 'Twi', 'or': 'Orija', 'xh': 'Xhosa', 'co': 'Korsisch', 'ca': 'Katalanisch', 'cy': 'Walisisch', 'cs': 'Tschechisch', 'zap': 'Zapotekisch', 'ps': 'Afghanisch (Paschtu)', 'pt': 'Portugiesisch', 'tl': 'Tagalog', 'pa': 'Pandschabisch', 'vi': 'Vietnamesisch', 'pl': 'Polnisch', 'hy': 'Armenisch', 'hr': 'Kroatisch', 'iu': 'Inukitut', 'hu': 'Ungarisch', 'hi': 'Hindi', 'ha': 'Hausa', 'he': u'Hebr\xe4isch', 'mg': 'Madagassisch', 'uz': 'Usbekisch', 'ml': 'Malayalam', 'mo': 'Moldauisch', 'mn': 'Mongolisch', 'mi': 'Maori', 'ik': 'Inupiak', 'mk': 'Mazedonisch', 'ur': 'Urdu', 'mt': 'Maltesisch', 'ms': 'Malaiisch', 'mr': 'Marathi', 'ug': 'Uigur', 'ta': 'Tamilisch', 'my': 'Burmesisch', 'aa': 'Afar', 'ab': 'Abchasisch', 'ss': 'Swazi', 'af': 'Afrikaans', 'tn': 'Setswana', 'sw': 'Suaheli', 'is': u'Isl\xe4ndisch', 'am': 'Amharisch', 'it': 'Italienisch', 'sv': 'Schwedisch', 'ia': 'Interlingua', 'as': 'Assamesisch', 'ar': 'Arabisch', 'su': 'Sudanesisch', 'zu': 'Zulu', 'ay': 'Aymara', 'az': 'Aserbaidschanisch', 'ie': 'Interlingue', 'id': 'Indonesisch', 'sk': 'Slowakisch', 'nl': u'Niederl\xe4ndisch', 'nn': 'Norwegisch Nynorsk', 'no': 'Norwegisch', 'na': 'Nauruisch', 'nb': u'Norwegisch Bokm\xe5l', 'ne': 'Nepalesisch', 'vo': u'Volap\xfck', 'fr': u'Franz\xf6sisch', 'sm': 'Samoanisch', 'fy': 'Friesisch', 'fa': 'Persisch', 'fi': 'Finnisch', 'fj': 'Fidschianisch', 'sa': 'Sanskrit', 'fo': u'F\xe4r\xf6isch', 'ka': 'Georgisch', 'kk': 'Kasachisch', 'sr': 'Serbisch', 'sq': 'Albanisch', 'ko': 'Koreanisch', 'kn': 'Kannada', 'km': 'Kambodschanisch', 'kl': u'Gr\xf6nl\xe4ndisch', 'ks': 'Kaschmirisch', 'si': 'Singhalesisch', 'sh': 'Serbo-Kroatisch', 'so': 'Somali', 'sn': 'Shona', 'ku': 'Kurdisch', 'sl': 'Slowenisch', 'ky': 'Kirgisisch', 'sg': 'Sango', 'sd': 'Sindhi'}

countries={'BD': 'Bangladesh', 'RW': 'Rwanda', 'DJ': 'Djibouti', 'BN': 'Brunei', 'MH': 'Marshall-Inseln', 'ST': u'Sao Tom\xe9 und Principe', 'BW': 'Botswana', 'GB': 'Grossbritannien', 'SB': 'Salomon-Inseln', 'ZW': 'Zimbabwe', 'CV': 'Kapverden'}

months=['Januar', 'Februar', u'M\xe4rz', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember']

abbrMonths=['Jan', 'Feb', 'Mrz', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez']

days=['Montag', 'Dienstag', 'Mittwoch', 'Donnerstag', 'Freitag', 'Samstag', 'Sonntag']

abbrDays=['Mo', 'Di', 'Mi', 'Do', 'Fr', 'Sa', 'So']

dateFormats={'medium': '%d.%m.%Y', 'full': '%%(dayname)s, %d. %%(monthname)s %Y', 'long': '%d. %%(monthname)s %Y', 'short': '%d.%m.%y'}

numericSymbols={'group': "'", 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
