# Formatting configuration for locale da_DK

languages={'el': u'Gr\xe6sk', 'gu': 'Gujaratisk', 'en': 'Engelsk', 'zh': 'Kinesisk', 'vi': 'Vietnamesisk', 'ca': 'Katalansk', 'am': 'Amharisk', 'it': 'Italiensk', 'cs': 'Tjekkisk', 'ar': 'Arabisk', 'ga': 'Irsk', 'eu': 'Baskisk', 'et': 'Estisk', 'gl': 'Galicisk', 'id': 'Indonesisk', 'es': 'Spansk', 'ru': 'Russisk', 'az': 'Aserbajdsjan', 'nl': 'Hollandsk', 'pt': 'Portugisisk', 'no': 'Norsk', 'tr': 'Tyrkisk', 'lv': 'Lettisk', 'lt': 'Litauisk', 'th': 'Thailandsk', 'so': 'Somalisk', 'he': 'Hebraisk', 'is': 'Islandsk', 'pl': 'Polsk', 'haw': 'Hawaii', 'be': 'Hviderussisk', 'fr': 'Fransk', 'bg': 'Bulgarsk', 'uk': 'Ukrainsk', 'sl': 'Slovensk', 'hr': 'Kroatisk', 'bn': 'Bengalsk', 'de': 'Tysk', 'da': 'Dansk', 'fa': 'Persisk', 'fi': 'Finsk', 'hy': 'Armensk', 'hu': 'Ungarsk', 'ja': 'Japansk', 'fo': u'F\xe6r\xf8sk', 'ka': 'Georgisk', 'ro': u'Rum\xe6nsk', 'kk': 'Kasakhisk', 'sr': 'Serbisk', 'sq': 'Albansk', 'ko': 'Koreansk', 'sv': 'Svensk', 'mk': 'Makedonsk', 'sk': 'Slovakisk', 'mt': 'Maltesisk', 'sh': 'Serbokroatisk', 'kw': 'Cornisk', 'ku': 'Kurdisk', 'mr': 'Marathisk', 'ta': 'Tamilsk'}

countries={'BD': 'Bangladesh', 'BE': 'Belgien', 'BF': 'Burkina Faso', 'BG': 'Bulgarien', 'BA': 'Bosnien-Hercegovina', 'BB': 'Barbados', 'WF': 'Wallis og Futuna', 'BM': 'Bermuda', 'BN': 'Brunei Darussalam', 'BO': 'Bolivia', 'BH': 'Bahrain', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaica', 'BV': 'Bouvet Island', 'BW': 'Botswana', 'WS': 'Samoa', 'BR': 'Brasilien', 'BS': 'Bahamas', 'BY': 'Hviderusland', 'BZ': 'Belize', 'RU': 'Rusland', 'RW': 'Rwanda', 'TL': u'\xd8sttimor', 'RE': 'Reunion', 'TM': 'Turkmenistan', 'TJ': 'Tadsjikistan', 'RO': u'Rum\xe6nien', 'TK': 'Tokelau', 'GW': 'Guinea-Bissau', 'GU': 'Guam', 'GT': 'Guatemala', 'GS': u'South Georgia og De Sydlige Sandwich\xf8er', 'GR': u'Gr\xe6kenland', 'GQ': u'\xc6kvatorialguinea', 'GP': 'Guadeloupe', 'JP': 'Japan', 'GY': 'Guyana', 'GF': 'Fransk Guyana', 'GE': 'Georgien', 'GD': 'Grenada', 'GB': 'Storbritannien', 'GA': 'Gabon', 'SV': 'El Salvador', 'GN': 'Guinea', 'GM': 'Gambia', 'GL': u'Gr\xf8nland', 'GI': 'Gibraltar', 'GH': 'Ghana', 'OM': 'Oman', 'TN': 'Tunesien', 'JO': 'Jordan', 'SP': 'Serbia', 'HR': 'Kroatien', 'HT': 'Haiti', 'HU': 'Ungarn', 'HK': 'Hong Kong S.A.R., China', 'HN': 'Honduras', 'HM': 'Heard Island og McDonald Islands', 'VE': 'Venezuela', 'PR': 'Puerto Rico', 'PS': u'De pal\xe6stinensiske omr\xe5der', 'PW': 'Palau', 'PT': 'Portugal', 'SJ': 'Svalbard og Jan Mayen', 'PY': 'Paraguay', 'IQ': 'Irak', 'PA': 'Panama', 'PF': 'Fransk Polynesien', 'PG': 'Papua Ny Guinea', 'PE': 'Peru', 'PK': 'Pakistan', 'PH': 'Filippinerne', 'PN': 'Pitcairn', 'PL': 'Polen', 'PM': 'Saint Pierre og Miquelon', 'ZM': 'Zambia', 'EH': 'Vestsahara', 'EE': 'Estland', 'EG': 'Egypten', 'ZA': 'Sydafrika', 'EC': 'Ecuador', 'IT': 'Italien', 'VN': 'Vietnam', 'SB': u'Salomon\xf8erne', 'ET': 'Etiopien', 'SO': 'Somalia', 'ZW': 'Zimbabwe', 'SA': 'Saudi-Arabien', 'ES': 'Spanien', 'ER': 'Eritrea', 'MD': 'Republikken Moldova', 'MG': 'Madagaskar', 'MA': 'Marokko', 'MC': 'Monaco', 'UZ': 'Usbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MO': 'Macao S.A.R. China', 'MN': 'Mongoliet', 'MH': u'Marshall\xf8erne', 'MK': 'Republikken Makedonien', 'MU': 'Mauritius', 'MT': 'Malta', 'MW': 'Malawi', 'MV': 'Maldiverne', 'MQ': 'Martinique', 'MP': 'Nordmarianerne', 'MS': 'Montserrat', 'MR': 'Mauretanien', 'UG': 'Uganda', 'MY': 'Malaysia', 'MX': 'Mexico', 'IL': 'Israel', 'FR': 'Frankrig', 'IO': 'Det Britiske Territorium i Det Indiske Ocean', 'SH': 'St. Helena', 'FI': 'Finland', 'FJ': 'Fiji', 'FK': u'Falklands\xf8erne', 'FM': 'Mikronesiens Forenede Stater', 'FO': u'F\xe6r\xf8erne', 'NI': 'Nicaragua', 'NL': 'Holland', 'NO': 'Norge', 'NA': 'Namibia', 'VU': 'Vanuatu', 'NC': 'Ny Caledonien', 'NE': 'Niger', 'NF': 'Norfolk Island', 'NG': 'Nigeria', 'NZ': 'New Zealand', 'NP': 'Nepal', 'NR': 'Nauru', 'NU': 'Niue', 'CK': 'Cook Islands', 'CI': 'Elfenbenskysten', 'CH': 'Schweiz', 'CO': 'Colombia', 'CN': 'Kina', 'CM': 'Cameroun', 'CL': 'Chile', 'CC': 'Cocos (Keeling) Islands', 'CA': 'Canada', 'CG': 'Congo', 'CF': 'Centralafrikanske Republik', 'CD': 'Den Demokratiske Republik Congo', 'CZ': 'Tjekkiet', 'CY': 'Cypern', 'CX': u'Jule\xf8en', 'CR': 'Costa Rica', 'Fallback': 'en', 'CV': 'Kap Verde', 'CU': 'Cuba', 'SZ': 'Swaziland', 'SY': 'Syrien', 'KG': 'Kirgisistan', 'KE': 'Kenya', 'SR': 'Surinam', 'KI': 'Kiribati', 'KH': 'Cambodja', 'KN': 'Saint Kitts og Nevis', 'KM': 'Comorerne', 'ST': 'Sao Tome og Principe', 'SK': 'Slovakiet', 'KR': 'Sydkorea', 'SI': 'Slovenien', 'KP': 'Nordkorea', 'KW': 'Kuwait', 'SN': 'Senegal', 'SM': 'San Marino', 'SL': 'Sierra Leone', 'SC': 'Seychellerne', 'KZ': 'Kasakhstan', 'KY': u'Cayman\xf8erne', 'SG': 'Singapore', 'SE': 'Sverige', 'SD': 'Sudan', 'DO': 'Den Dominikanske Republik', 'DM': 'Dominica', 'DJ': 'Djibouti', 'DK': 'Danmark', 'VG': u'De britiske jomfru\xf8er', 'DE': 'Tyskland', 'YE': 'Yemen', 'DZ': 'Algeriet', 'US': 'Amerikas Forenede Stater', 'UY': 'Uruguay', 'YU': 'Jugoslavien', 'YT': 'Mayotte', 'UM': u'De Mindre Amerikanske Overs\xf8iske \xd8er', 'LB': 'Libanon', 'LC': 'Saint Lucia', 'LA': 'Laos', 'TV': 'Tuvalu', 'TW': 'Taiwan', 'TT': 'Trinidad og Tobago', 'TR': 'Tyrkiet', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': 'Letland', 'TO': 'Tonga', 'LT': 'Litauen', 'LU': 'Luxembourg', 'LR': 'Liberia', 'LS': 'Lesotho', 'TH': 'Thailand', 'TF': 'Franske Besiddelser i Det Sydlige Indiske Ocean', 'TG': 'Togo', 'TD': 'Tchad', 'TC': 'Turks and Caicos Islands', 'LY': 'Libyen', 'VA': 'Vatikanstaten', 'VC': 'St. Vincent og Grenadinerne', 'AE': 'Forenede Arabiske Emirater', 'AD': 'Andorra', 'AG': 'Antigua og Barbuda', 'AF': 'Afghanistan', 'AI': 'Anguilla', 'VI': u'De amerikanske jomfru\xf8er', 'IS': 'Island', 'IR': 'Iran', 'AM': 'Armenien', 'AL': 'Albanien', 'AO': 'Angola', 'AN': 'Hollandske Antiller', 'AQ': 'Antarktis', 'AS': 'Amerikansk Samoa', 'AR': 'Argentina', 'AU': 'Australien', 'AT': u'\xd8strig', 'AW': 'Aruba', 'IN': 'Indien', 'TZ': 'Tanzania', 'AZ': 'Aserbajdsjan', 'IE': 'Irland', 'ID': 'Indonesien', 'UA': 'Ukraine', 'QA': 'Qatar', 'MZ': 'Mozambique'}

months=['januar', 'februar', 'marts', 'april', 'maj', 'juni', 'juli', 'august', 'september', 'oktober', 'november', 'december']

abbrMonths=['jan', 'feb', 'mar', 'apr', 'maj', 'jun', 'jul', 'aug', 'sep', 'okt', 'nov', 'dec']

days=['mandag', 'tirsdag', 'onsdag', 'torsdag', 'fredag', u'l\xf8rdag', u's\xf8ndag']

abbrDays=['man', 'tir', 'ons', 'tor', 'fre', u'l\xf8r', u's\xf8n']

dateFormats={'medium': '%d-%m-%Y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d. %%(abbrmonthname)s %Y', 'short': '%d-%m-%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
