# Formatting configuration for locale de_AT

languages={'gv': 'Manx', 'gu': 'Gujarati', 'gd': u'Schottisch-G\xe4lisch', 'ga': 'Irisch', 'gn': 'Guarani', 'gl': 'Galizisch', 'la': 'Latein', 'ln': 'Lingala', 'lo': 'Laotisch', 'tt': 'Tatarisch', 'tr': u'T\xfcrkisch', 'ts': 'Tsonga', 'lv': 'Lettisch', 'to': 'Tongaisch', 'lt': 'Litauisch', 'tk': 'Turkmenisch', 'th': 'Thai', 'ti': 'Tigrinja', 'tg': 'Tadschikisch', 'te': 'Telugu', 'haw': 'Hawaiianisch', 'yi': 'Jiddisch', 'yo': 'Joruba', 'de': 'Deutsch', 'da': u'D\xe4nisch', 'dz': 'Bhutanisch', 'st': 'Sotho', 'qu': 'Quechua', 'el': 'Griechisch', 'eo': 'Esperanto', 'en': 'Englisch', 'zh': 'Chinesisch', 'za': 'Zhuang', 'uk': 'Ukrainisch', 'eu': 'Baskisch', 'et': 'Estnisch', 'es': 'Spanisch', 'ru': 'Russisch', 'rw': 'Ruandisch', 'rm': u'R\xe4toromanisch', 'rn': 'Kirundi', 'ro': u'Rum\xe4nisch', 'bn': 'Bengalisch', 'be': u'Wei\xdfrussisch', 'bg': 'Bulgarisch', 'ba': 'Baschkirisch', 'wo': 'Wolof', 'jv': 'Javanisch', 'bo': 'Tibetisch', 'bh': 'Biharisch', 'bi': 'Bislama', 'br': 'Bretonisch', 'ja': 'Japanisch', 'om': 'Oromo', 'oc': 'Okzitanisch', 'tw': 'Twi', 'or': 'Orija', 'xh': 'Xhosa', 'co': 'Korsisch', 'ca': 'Katalanisch', 'cy': 'Walisisch', 'cs': 'Tschechisch', 'zap': 'Zapotekisch', 'ps': 'Afghanisch (Paschtu)', 'pt': 'Portugiesisch', 'tl': 'Tagalog', 'pa': 'Pandschabisch', 'vi': 'Vietnamesisch', 'pl': 'Polnisch', 'hy': 'Armenisch', 'hr': 'Kroatisch', 'iu': 'Inukitut', 'hu': 'Ungarisch', 'hi': 'Hindi', 'ha': 'Hausa', 'he': u'Hebr\xe4isch', 'mg': 'Madagassisch', 'uz': 'Usbekisch', 'ml': 'Malayalam', 'mo': 'Moldauisch', 'mn': 'Mongolisch', 'mi': 'Maori', 'ik': 'Inupiak', 'mk': 'Mazedonisch', 'ur': 'Urdu', 'mt': 'Maltesisch', 'ms': 'Malaiisch', 'mr': 'Marathi', 'ug': 'Uigur', 'ta': 'Tamilisch', 'my': 'Burmesisch', 'aa': 'Afar', 'ab': 'Abchasisch', 'ss': 'Swazi', 'af': 'Afrikaans', 'tn': 'Setswana', 'sw': 'Suaheli', 'is': u'Isl\xe4ndisch', 'am': 'Amharisch', 'it': 'Italienisch', 'sv': 'Schwedisch', 'ia': 'Interlingua', 'as': 'Assamesisch', 'ar': 'Arabisch', 'su': 'Sudanesisch', 'zu': 'Zulu', 'ay': 'Aymara', 'az': 'Aserbaidschanisch', 'ie': 'Interlingue', 'id': 'Indonesisch', 'sk': 'Slowakisch', 'nl': u'Niederl\xe4ndisch', 'nn': 'Norwegisch Nynorsk', 'no': 'Norwegisch', 'na': 'Nauruisch', 'nb': u'Norwegisch Bokm\xe5l', 'ne': 'Nepalesisch', 'vo': u'Volap\xfck', 'fr': u'Franz\xf6sisch', 'sm': 'Samoanisch', 'fy': 'Friesisch', 'fa': 'Persisch', 'fi': 'Finnisch', 'fj': 'Fidschianisch', 'sa': 'Sanskrit', 'fo': u'F\xe4r\xf6isch', 'ka': 'Georgisch', 'kk': 'Kasachisch', 'sr': 'Serbisch', 'sq': 'Albanisch', 'ko': 'Koreanisch', 'kn': 'Kannada', 'km': 'Kambodschanisch', 'kl': u'Gr\xf6nl\xe4ndisch', 'ks': 'Kaschmirisch', 'si': 'Singhalesisch', 'sh': 'Serbo-Kroatisch', 'so': 'Somali', 'sn': 'Shona', 'ku': 'Kurdisch', 'sl': 'Slowenisch', 'ky': 'Kirgisisch', 'sg': 'Sango', 'sd': 'Sindhi'}

countries={'BD': 'Bangladesch', 'BE': 'Belgien', 'BF': 'Burkina Faso', 'BG': 'Bulgarien', 'BA': 'Bosnien und Herzegowina', 'BB': 'Barbados', 'WF': 'Wallis und Futuna', 'BM': 'Bermuda', 'BN': 'Brunei Darussalam', 'BO': 'Bolivien', 'BH': 'Bahrain', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaika', 'BV': 'Bouvetinsel', 'BW': 'Botsuana', 'WS': 'Samoa', 'BR': 'Brasilien', 'BS': 'Bahamas', 'BY': 'Belarus', 'BZ': 'Belize', 'RU': 'Russland', 'RW': 'Ruanda', 'TL': 'Osttimor', 'RE': u'R\xe9union', 'TM': 'Turkmenistan', 'TJ': 'Tadschikistan', 'RO': u'Rum\xe4nien', 'TK': 'Tokelau', 'GW': 'Guinea-Bissau', 'GU': 'Guam', 'GT': 'Guatemala', 'GS': u'S\xfcdgeorgien und die S\xfcdlichen Sandwichinseln', 'GR': 'Griechenland', 'GQ': u'\xc4quatorialguinea', 'GP': 'Guadeloupe', 'JP': 'Japan', 'GY': 'Guyana', 'GF': u'Franz\xf6sisch-Guayana', 'GE': 'Georgien', 'GD': 'Grenada', 'GB': u'Vereinigtes K\xf6nigreich', 'GA': 'Gabun', 'SV': 'El Salvador', 'GN': 'Guinea', 'GM': 'Gambia', 'GL': u'Gr\xf6nland', 'GI': 'Gibraltar', 'GH': 'Ghana', 'OM': 'Oman', 'TN': 'Tunesien', 'JO': 'Jordanien', 'SP': 'Serbien', 'HR': 'Kroatien', 'HT': 'Haiti', 'HU': 'Ungarn', 'HK': 'Hong Kong S.A.R., China', 'HN': 'Honduras', 'HM': 'Heard und McDonaldinseln', 'VE': 'Venezuela', 'PR': 'Puerto Rico', 'PS': u'Pal\xe4stina', 'PW': 'Palau', 'PT': 'Portugal', 'SJ': 'Svalbard und Jan Mayen', 'PY': 'Paraguay', 'IQ': 'Irak', 'PA': 'Panama', 'PF': u'Franz\xf6sisch-Polynesien', 'PG': 'Papua-Neuguinea', 'PE': 'Peru', 'PK': 'Pakistan', 'PH': 'Philippinen', 'PN': 'Pitcairn', 'PL': 'Polen', 'PM': 'St. Pierre und Miquelon', 'ZM': 'Sambia', 'EH': 'Westsahara', 'EE': 'Estland', 'EG': u'\xc4gypten', 'ZA': u'S\xfcdafrika', 'EC': 'Ecuador', 'IT': 'Italien', 'VN': 'Vietnam', 'SB': 'Salomonen', 'ET': u'\xc4thiopien', 'SO': 'Somalia', 'ZW': 'Simbabwe', 'SA': 'Saudi-Arabien', 'ES': 'Spanien', 'ER': 'Eritrea', 'MD': 'Moldawien', 'MG': 'Madagaskar', 'MA': 'Marokko', 'MC': 'Monaco', 'UZ': 'Usbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MO': 'Macau S.A.R.,China', 'MN': 'Mongolei', 'MH': 'Marshallinseln', 'MK': 'Mazedonien', 'MU': 'Mauritius', 'MT': 'Malta', 'MW': 'Malawi', 'MV': 'Malediven', 'MQ': 'Martinique', 'MP': u'N\xf6rdliche Marianen', 'MS': 'Montserrat', 'MR': 'Mauretanien', 'UG': 'Uganda', 'MY': 'Malaysia', 'MX': 'Mexiko', 'IL': 'Israel', 'FR': 'Frankreich', 'IO': 'Britisches Territorium im Indischen Ozean', 'SH': 'St. Helena', 'FI': 'Finnland', 'FJ': 'Fidschi', 'FK': 'Falklandinseln', 'FM': 'Mikronesien', 'FO': u'F\xe4r\xf6er', 'NI': 'Nicaragua', 'NL': 'Niederlande', 'NO': 'Norwegen', 'NA': 'Namibia', 'VU': 'Vanuatu', 'NC': 'Neukaledonien', 'NE': 'Niger', 'NF': 'Norfolkinsel', 'NG': 'Nigeria', 'NZ': 'Neuseeland', 'NP': 'Nepal', 'NR': 'Nauru', 'NU': 'Niue', 'CK': 'Cookinseln', 'CI': u"C\xf4te d'Ivoire", 'CH': 'Schweiz', 'CO': 'Kolumbien', 'CN': 'China', 'CM': 'Kamerun', 'CL': 'Chile', 'CC': 'Kokosinseln (Keeling)', 'CA': 'Kanada', 'CG': 'Kongo', 'CF': 'Zentralafrikanische Republik', 'CD': 'Demokratische Republik Kongo', 'CZ': 'Tschechien', 'CY': 'Zypern', 'CX': 'Weihnachtsinsel', 'CR': 'Costa Rica', 'Fallback': 'en', 'CV': 'Kap Verde', 'CU': 'Kuba', 'SZ': 'Swasiland', 'SY': 'Syrien', 'KG': 'Kirgisistan', 'KE': 'Kenia', 'SR': 'Suriname', 'KI': 'Kiribati', 'KH': 'Kambodscha', 'KN': 'St. Kitts und Nevis', 'KM': 'Komoren', 'ST': u'S\xe3o Tom\xe9 und Pr\xedncipe', 'SK': 'Slowakei', 'KR': 'Republik Korea', 'SI': 'Slowenien', 'KP': 'Demokratische Volksrepublik Korea', 'KW': 'Kuwait', 'SN': 'Senegal', 'SM': 'San Marino', 'SL': 'Sierra Leone', 'SC': 'Seychellen', 'KZ': 'Kasachstan', 'KY': 'Kaimaninseln', 'SG': 'Singapur', 'SE': 'Schweden', 'SD': 'Sudan', 'DO': 'Dominikanische Republik', 'DM': 'Dominica', 'DJ': 'Dschibuti', 'DK': u'D\xe4nemark', 'VG': 'Britische Jungferninseln', 'DE': 'Deutschland', 'YE': 'Jemen', 'DZ': 'Algerien', 'US': 'Vereinigte Staaten', 'UY': 'Uruguay', 'YU': 'Jugoslawien', 'YT': 'Mayotte', 'UM': 'Amerikanisch-Ozeanien', 'LB': 'Libanon', 'LC': 'St. Lucia', 'LA': 'Laos', 'TV': 'Tuvalu', 'TW': 'Taiwan', 'TT': 'Trinidad und Tobago', 'TR': u'T\xfcrkei', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': 'Lettland', 'TO': 'Tonga', 'LT': 'Litauen', 'LU': 'Luxemburg', 'LR': 'Liberia', 'LS': 'Lesotho', 'TH': 'Thailand', 'TF': u'Franz\xf6sische S\xfcd- und Antarktisgebiete', 'TG': 'Togo', 'TD': 'Tschad', 'TC': 'Turks- und Caicosinseln', 'LY': 'Libyen', 'VA': 'Vatikanstadt', 'VC': 'St. Vincent und die Grenadinen', 'AE': 'Vereinigte Arabische Emirate', 'AD': 'Andorra', 'AG': 'Antigua und Barbuda', 'AF': 'Afghanistan', 'AI': 'Anguilla', 'VI': 'Amerikanische Jungferninseln', 'IS': 'Island', 'IR': 'Iran', 'AM': 'Armenien', 'AL': 'Albanien', 'AO': 'Angola', 'AN': u'Niederl\xe4ndische Antillen', 'AQ': 'Antarktis', 'AS': 'Amerikanisch-Samoa', 'AR': 'Argentinien', 'AU': 'Australien', 'AT': u'\xd6sterreich', 'AW': 'Aruba', 'IN': 'Indien', 'TZ': 'Tansania', 'AZ': 'Aserbaidschan', 'IE': 'Irland', 'ID': 'Indonesien', 'UA': 'Ukraine', 'QA': 'Katar', 'MZ': 'Mosambik'}

months=[u'J\xe4nner', 'Februar', u'M\xe4rz', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember']

abbrMonths=[u'J\xe4n', 'Feb', u'M\xe4r', 'Apr', 'Mai', 'Jun', 'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez']

days=['Montag', 'Dienstag', 'Mittwoch', 'Donnerstag', 'Freitag', 'Samstag', 'Sonntag']

abbrDays=['Mo', 'Di', 'Mi', 'Do', 'Fr', 'Sa', 'So']

dateFormats={'medium': '%d.%m.%Y', 'full': '%%(dayname)s, %d. %%(monthname)s %Y', 'long': '%d. %%(monthname)s %Y', 'short': '%d.%m.%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
