/*
	WWW SQL Designer, (C) 2005 Ondra Zara, o.z.fw@seznam.cz

    This file is part of WWW SQL Designer.

    WWW SQL Designer is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    WWW SQL Designer is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with WWW SQL Designer; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA	
*/
var DELAY = 10; /* pauza mezi rostouci animaci */
var BAR_HEIGHT = 100;

var TYPE_TABLE = 0;
var TYPE_TITLE = 1;
var TYPE_ROW = 2;
var TYPE_ROWTITLE = 3;
var TYPE_RELATION = 4;
var TYPE_RELATION_PART = 5; /* not used */
var TYPE_BAR = 6;
var TYPE_MAP = 7;
