h1 {
	font-size: 2em;
	color: #4B4545;
	text-align: center;
}

input.textbox {
	font-family: Monaco, "lucida console", Courier;
	border: 1px solid #CCCCCC;
	padding: 2px 4px;
	margin-top: .3em;
}

#interpreter_area {
	display: block;
	border: 1px solid #CCCCCC;
	padding: 2px 4px;
	margin-top: .3em;
	width: 600px;
	height: 300px;
	overflow: auto;
}

#interpreter_output {
	display: inline;
	font-family: Monaco, "lucida console", Courier;
}

#interpreter_output span {
	white-space: -moz-pre-wrap; /* Mozilla */
	white-space: -o-pre-wrap; /* Opera 7 */
	white-space: pre-wrap; /* CSS 2.1 */
	white-space: pre-line; /* CSS 3 (and 2.1 as well, actually) */
	word-wrap: break-word; /* IE */
	wrap-option: emergency; /* CSS 3 */
}

#interpreter_block_text {
	height: 200px;
	width: 500px;
	font-family: Monaco, "lucida console", Courier;
	border: 1px solid #CCCCCC;
}

#prompt {
	float: left;
	font-family: Monoco, "lucida console", Courier;
	vertical-align: bottom;
	margin-top: .6em;
}

.textbox:focus {
	background-color: #FFFEE3;
}

.code {	color: blue; }
.data { color: black; }
.error { color: red; }
.banner { color: green; }
.invisible { display: none; } 
