# Formatting configuration for locale sq

languages={'sq': 'shqipe'}

countries={'AL': u'Shqip\xebria'}

months=['janar', 'shkurt', 'mars', 'prill', 'maj', 'qershor', 'korrik', 'gusht', 'shtator', 'tetor', u'n\xebntor', 'dhjetor']

abbrMonths=['Jan', 'Shk', 'Mar', 'Pri', 'Maj', 'Qer', 'Kor', 'Gsh', 'Sht', 'Tet', u'N\xebn', 'Dhj']

days=[u'e h\xebn\xeb', u'e mart\xeb', u'e m\xebrkur\xeb', 'e enjte', 'e premte', u'e shtun\xeb', 'e diel']

abbrDays=[u'H\xebn', 'Mar', u'M\xebr', 'Enj', 'Pre', 'Sht', 'Die']

dateFormats={'medium': '%Y-%m-%d', 'full': '%%(dayname)s, %d %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%y-%m-%d'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
