# Formatting configuration for locale id_ID

languages={'gv': 'Manx', 'gu': 'Gujarati', 'alg': 'Bahasa Algonquia', 'afa': 'Afro-Asiatik (Lainnya)', 'gd': 'Gaelik Skotlandia', 'ga': 'Irlandia', 'gn': 'Guarani', 'gl': 'Gallegan', 'lg': 'Ganda', 'gmh': 'Jerman, Abad Pertengahan (kl.1050-1500)', 'lb': 'Luxembourg', 'la': 'Latin', 'ln': 'Lingala', 'lo': 'Lao', 'tt': 'Tatar', 'tr': 'Turkish', 'ts': 'Tsonga', 'li': 'Limburg', 'lv': 'Latvian', 'aus': 'Bahasa-bahasa Australia', 'lu': 'Luba-Katanga', 'tk': 'Turkmen', 'th': 'Thai', 'sr': 'Serbian', 'tg': 'Tajik', 'te': 'Telugu', 'grc': 'Yunani Kuno (sd 1453)', 'yi': 'Yiddish', 'mga': 'Irlandia Abad Pertengahan (900-1200)', 'yo': 'Yoruba', 'de': 'Jerman', 'da': 'Denmark', 'za': 'Zhuang', 'ta': 'Tamil', 'dz': 'Dzongkha', 'rm': 'Rhaeto-Romance', 'dv': 'Divehi', 'gem': 'Jermanik (Lainnya)', 'bat': 'Baltik (Lainnya)', 'tl': 'Tagalog', 'qu': 'Quechua', 'ml': 'Malayalam', 'ban': 'Balin', 'bai': 'Bahasa-bahasa Bamileke', 'el': 'Yunani', 'eo': 'Esperanto', 'en': 'Inggris', 'art': 'Buatan (Lainnya)', 'ee': 'Ewe', 'phi': 'Filipina (Lainnya)', 'mh': 'Marshall', 'arc': 'Aram', 'mk': 'Macedonian', 'eu': 'Basque', 'et': 'Estonian', 'arn': 'Araucan', 'ru': 'Russian', 'rw': 'Kinyarwanda', 'goh': 'Jerman Kuno (kl.750-1050)', 'peo': 'Persia Kuno (kl.600-400 SM.)', 'got': 'Gothik', 'rn': 'Rundi', 'ro': 'Romanian', 'jv': 'Jawa', 'sux': 'Sumeria', 'be': 'Belarusia', 'bg': 'Bulgaria', 'ba': 'Bashkir', 'wa': 'Walloon', 'ast': 'Astur', 'wo': 'Wolof', 'bm': 'Bambara', 'bn': 'Bengal', 'bo': 'Tibet', 'bh': 'Bihari', 'bi': 'Bislama', 'br': 'Breton', 'bs': 'Bosnia', 'lez': 'Lezghia', 'ja': 'Japanese', 'cop': 'Koptik', 'om': 'Oromo', 'oj': 'Ojibwa', 'ace': 'Aceh', 'syr': 'Syria', 'ty': 'Tahitian', 'ath': 'Bahasa-bahasa Athapaska', 'akk': 'Akkadien', 'tw': 'Twi', 'os': 'Ossetic', 'or': 'Oriya', 'xh': 'Xhosa', 'ch': 'Chamorro', 'co': 'Korsika', 'jrb': 'Judeo-Arab', 'ca': 'Catalan', 'ce': 'Chechen', 'cy': 'Welsh', 'ang': 'Inggris Kuno (kl.450-1100)', 'cs': 'Ceko', 'cr': 'Cree', 'apa': 'Bahasa-bahasa Apache', 'cv': 'Chuvash', 've': 'Venda', 'ps': 'Pashto (Pushto)', 'kok': 'Konkani', 'pt': 'Portugis', 'chk': 'Chuuke', 'fro': 'Perancis Kuno (842-kl.1400)', 'lt': 'Lithuania', 'frm': 'Perancis, Abad Pertengahan (kl.1400-1600)', 'mkh': 'Mon-Khmer (Lainnya)', 'paa': 'Papuan (Lainnya)', 'pa': 'Punjabi', 'vi': 'Vietnamese', 'kos': 'Kosrae', 'gil': 'Gilbert', 'is': 'Icelandic', 'pl': 'Polish', 'hz': 'Herero', 'zh': 'Cina', 'hy': 'Armenia', 'hr': 'Kroasia', 'ti': 'Tigrinya', 'ka': 'Georgian', 'hu': 'Hungaria', 'hi': 'Hindi', 'ho': 'Hiri Motu', 'ha': 'Hausa', 'bug': 'Bugis', 'he': 'Ibrani', 'mg': 'Malagasi', 'pi': 'Pali', 'fur': 'Friuli', 'uz': 'Uzbek', 'car': 'Karib', 'mo': 'Moldavian', 'mn': 'Mongolian', 'mi': 'Maori', 'ik': 'Inupiaq', 'cau': 'Kaukasia (Lainnya)', 'ur': 'Urdu', 'mt': 'Maltese', 'cai': 'India Amerika Tengah (Lainnnya)', 'ms': 'Malay', 'mr': 'Marathi', 'ug': 'Uighur', 'haw': 'Hawaii', 'my': 'Burma', 'sq': 'Albanian', 'aa': 'Afar', 'ab': 'Abkhaz', 'ae': 'Avestan', 'ss': 'Swati', 'af': 'Afrikaans', 'tn': 'Tswana', 'ne': 'Nepal', 'ak': 'Akan', 'am': 'Amharik', 'it': 'Italian', 'sv': 'Swedia', 'ii': 'Sichuan Yi', 'ia': 'Interlingua', 'as': 'Assam', 'ar': 'Arab', 'su': 'Sundan', 'io': 'Ido', 'av': 'Avarik', 'ay': 'Aymara', 'az': 'Azerbaijan', 'ie': 'Interlingue', 'id': 'Bahasa Indonesia', 'ig': 'Igbo', 'sk': 'Slovak', 'jpr': 'Judeo-Persia', 'nl': 'Belanda', 'nn': 'Norwegian Nynorsk', 'no': 'Norwegian', 'na': 'Nauru', 'nb': u'Norwegian Bokm\xe5l', 'enm': 'Inggris, Abad Pertengahan (1100-1500)', 'ng': 'Ndonga', 'ny': 'Nyanja; Chichewa; Chewa', 'vo': u'Volap\xfck', 'es': 'Spanyol', 'zu': 'Zulu', 'so': 'Somali', 'mis': 'Bahasa Lain-lain', 'nv': 'Navajo', 'sn': 'Shona', 'map': 'Austronesia', 'fr': 'Perancis', 'cel': 'Celtic (Lainnya)', 'sm': 'Samoan', 'fy': 'Frisi', 'fa': 'Persia', 'mad': 'Madura', 'ff': 'Fulah', 'fi': 'Finlandia', 'fj': 'Fiji', 'sa': 'Sanskrit', 'uk': 'Ukrainian', 'fo': 'Faro', 'mak': 'Makassar', 'egy': 'Mesir Kuno', 'kg': 'Kongo', 'kk': 'Kazakh', 'kj': 'Kuanyama', 'ki': 'Kikuyu', 'ko': 'Korea', 'kn': 'Kannada', 'km': 'Khmer', 'kl': 'Kalaallisut', 'ks': 'Kashmir', 'kr': 'Kanuri', 'si': 'Sinhalese', 'sh': 'Serbo-Croatian', 'kw': 'Cornish', 'kv': 'Komi', 'ku': 'Kurdi', 'sl': 'Slovenian', 'sc': 'Sardinian', 'ky': 'Kirghiz', 'sg': 'Sango', 'sw': 'Swahili', 'se': 'Northern Sami', 'sd': 'Sindhi'}

countries={'BD': 'Bangladesh', 'BE': 'Belgia', 'BF': 'Burkina Faso', 'BG': 'Bulgaria', 'BA': 'Bosnia dan Herzegovina', 'BB': 'Barbados', 'WF': 'Wallis dan Futuna', 'BM': 'Bermuda', 'BN': 'Brunei', 'BO': 'Bolivia', 'BH': 'Bahrain', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaika', 'BV': 'Kepulauan Bouvet', 'BW': 'Botswana', 'WS': 'Samoa', 'BR': 'Brazil', 'BS': 'Bahamas', 'BY': 'Belarusia', 'BZ': 'Belize', 'RU': 'Rusia', 'RW': 'Rwanda', 'RE': u'R\xe9union', 'TM': 'Turkmenistan', 'TJ': 'Tajikistan', 'RO': 'Romania', 'TK': 'Tokelau', 'GW': 'Guinea-Bissau', 'GU': 'Guam', 'GT': 'Guatemala', 'GS': 'Georgia Selatan dan Kepulauan Sandwich Selatan', 'GR': 'Yunani', 'GQ': 'Guinea Khatulistiwa', 'GP': 'Guadeloupe', 'JP': 'Jepang', 'GY': 'Guyana', 'GF': 'Guyana Perancis', 'GE': 'Georgia', 'GD': 'Grenada', 'GB': 'Inggris Raya', 'GA': 'Gabon', 'SV': 'El Salvador', 'GN': 'Guinea', 'GM': 'Gambia', 'GL': 'Greenland', 'GI': 'Gibraltar', 'GH': 'Ghana', 'OM': 'Oman', 'TN': 'Tunisia', 'JO': 'Yordania', 'SP': 'Serbia', 'HR': 'Kroasia', 'HT': 'Haiti', 'HU': 'Hungaria', 'HK': 'Hong Kong S.A.R., Cina', 'HN': 'Honduras', 'HM': 'Pulau Heard dan Kepulauan McDonald', 'VE': 'Venezuela', 'PR': 'Puerto Riko', 'PS': 'Otoritas Palestina', 'PW': 'Palau', 'PT': 'Portugis', 'SJ': 'Svalbard dan Jan Mayen', 'PY': 'Paraguay', 'IQ': 'Iraq', 'PA': 'Panama', 'PF': 'Polynesia Perancis', 'PG': 'Papua Nugini', 'PE': 'Peru', 'PK': 'Pakistan', 'PH': 'Filipina', 'PN': 'Pitcairn', 'PL': 'Polandia', 'PM': 'Saint Pierre dan Miquelon', 'ZM': 'Zambia', 'EH': 'Sahara Barat', 'EE': 'Estonia', 'EG': 'Mesir', 'ZA': 'Afrika Selatan', 'EC': 'Ekuador', 'IT': 'Itali', 'VN': 'Vietnam', 'SB': 'Kepulauan Solomon', 'ET': 'Ethiopia', 'SO': 'Somalia', 'ZW': 'Zimbabwe', 'SA': 'Arab Saudi', 'ES': 'Spanyol', 'ER': 'Eritrea', 'MD': 'Moldova', 'MG': 'Madagaskar', 'MA': 'Maroko', 'MC': 'Monaco', 'UZ': 'Uzbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MO': 'Makao S.A.R. Cina', 'MN': 'Mongolia', 'MH': 'Kepulauan Marshall', 'MK': 'Macedonia', 'MU': 'Mauritius', 'MT': 'Malta', 'MW': 'Malawi', 'MV': 'Maldives', 'MQ': 'Martinique', 'MP': 'Kepualuan Mariana Utara', 'MS': 'Montserrat', 'MR': 'Mauritania', 'UG': 'Uganda', 'MY': 'Malaysia', 'MX': 'Mexico', 'IL': 'Israel', 'FR': 'Perancis', 'SH': 'Saint Helena', 'FI': 'Finlandia', 'FJ': 'Fiji', 'FK': 'Kepulauan Falkland', 'FM': 'Mikronesia', 'FO': 'Kepulauan Faroe', 'NI': 'Nicaragua', 'NL': 'Netherlands', 'NO': 'Norwegia', 'NA': 'Namibia', 'VU': 'Vanuatu', 'NC': 'Kaledonia Baru', 'NE': 'Niger', 'NF': 'Kepulauan Norfolk', 'NG': 'Nigeria', 'NZ': 'Selandia Baru', 'NP': 'Nepal', 'NR': 'Nauru', 'NU': 'Niue', 'CK': 'Kepulauan Cook', 'CI': 'Pantai Gading', 'CH': 'Swiss', 'CO': 'Kolombia', 'CN': 'Cina', 'CM': 'Kamerun', 'CL': 'Chili', 'CC': 'Kepulauan Cocos', 'CA': 'Kanada', 'CG': 'Kongo', 'CF': 'Republik Afrika Tengah', 'CD': 'Republik Demokratik Kongo', 'CZ': 'Republik Ceko', 'CY': 'Siprus', 'CX': 'Pulau Christmas', 'CR': 'Kosta Rika', 'CV': 'Tanjung Verde', 'CU': 'Kuba', 'SZ': 'Swaziland', 'SY': 'Syria', 'KG': 'Kyrgyzstan', 'KE': 'Kenya', 'SR': 'Suriname', 'KI': 'Kiribati', 'KH': 'Kamboja', 'KN': 'Saint Kitts dan Nevis', 'KM': 'Komoros', 'ST': 'Sao Tome dan Principe', 'SK': 'Slovakia', 'KR': 'Korea Selatan', 'SI': 'Slovenia', 'KP': 'Korea Utara', 'KW': 'Kuwait', 'SN': 'Senegal', 'SM': 'San Marino', 'SL': 'Sierra Leone', 'SC': 'Seychelles', 'KZ': 'Kazakhstan', 'KY': 'Kepulauan Kayman', 'SG': 'Singapura', 'SE': 'Sweden', 'SD': 'Sudan', 'DO': 'Republik Dominika', 'DM': 'Dominika', 'DJ': 'Jibouti', 'DK': 'Denmark', 'VG': 'Kepulauan British Virgin', 'DE': 'Jerman', 'YE': 'Yaman', 'DZ': 'Algeria', 'US': 'Amerika Serikat', 'UY': 'Uruguay', 'YU': 'Yugoslavia', 'YT': 'Mayotte', 'LB': 'Lebanon', 'LC': 'Santa Lusia', 'LA': 'Laos', 'TV': 'Tuvalu', 'TW': 'Taiwan', 'TT': 'Trinidad dan Tobago', 'TR': 'Turkey', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': 'Latvia', 'TO': 'Tonga', 'LT': 'Lithuania', 'LU': 'Luxembourg', 'LR': 'Liberia', 'LS': 'Lesotho', 'TH': 'Thailand', 'TG': 'Togo', 'TD': 'Chad', 'LY': 'Libya', 'VA': 'Vatikan', 'VC': 'Saint Vincent dan Grenadines', 'AE': 'Uni Emirat Arab', 'AD': 'Andora', 'AG': 'Antigua dan Barbuda', 'AF': 'Afghanistan', 'AI': 'Anguilla', 'VI': 'Kepulauan U.S. Virgin', 'IS': 'Islandia', 'IR': 'Iran', 'AM': 'Armenia', 'AL': 'Albania', 'AO': 'Angola', 'AN': 'Antilles Belanda', 'AQ': 'Antarktika', 'AS': 'Samoa Amerika', 'AR': 'Argentina', 'AU': 'Australia', 'AT': 'Austria', 'AW': 'Aruba', 'IN': 'India', 'TZ': 'Tanzania', 'AZ': 'Azerbaijan', 'IE': 'Irlandia', 'ID': 'Indonesia', 'UA': 'Ukraina', 'QA': 'Qatar', 'MZ': 'Mozambique'}

months=['Januari', 'Februari', 'Maret', 'April', 'Mei', 'Juni', 'Juli', 'Agustus', 'September', 'Oktober', 'November', 'Desember']

abbrMonths=['Jan', 'Feb', 'Mar', 'Apr', 'Mei', 'Jun', 'Jul', 'Agu', 'Sep', 'Okt', 'Nov', 'Des']

days=['Senin', 'Selasa', 'Rabu', 'Kamis', 'Jumat', 'Sabtu', 'Minggu']

abbrDays=['Sen', 'Sel', 'Rab', 'Kam', 'Jum', 'Sab', 'Min']

dateFormats={'medium': '%d %%(abbrmonthname)s %y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
