# Formatting configuration for locale gu

languages={'gu': u'\u0a97\u0ac1\u0a9c\u0ab0\u0abe\u0aa4\u0ac0'}

countries={'IN': u'\u0aad\u0abe\u0ab0\u0aa4'}

months=[u'\u0a9c\u0abe\u0aa8\u0acd\u0aaf\u0ac1\u0a86\u0ab0\u0ac0', u'\u0aab\u0ac7\u0aac\u0acd\u0ab0\u0ac1\u0a86\u0ab0\u0ac0', u'\u0aae\u0abe\u0ab0\u0acd\u0a9a', u'\u0a8f\u0aaa\u0acd\u0ab0\u0abf\u0ab2', u'\u0aae\u0ac7', u'\u0a9c\u0ac2\u0aa8', u'\u0a9c\u0ac1\u0ab2\u0abe\u0a88', u'\u0a91\u0a97\u0ab8\u0acd\u0a9f', u'\u0ab8\u0aaa\u0acd\u0a9f\u0ac7\u0aae\u0acd\u0aac\u0ab0', u'\u0a91\u0a95\u0acd\u0a9f\u0acd\u0aac\u0ab0', u'\u0aa8\u0ab5\u0ac7\u0aae\u0acd\u0aac\u0ab0', u'\u0aa1\u0abf\u0ab8\u0ac7\u0aae\u0acd\u0aac\u0ab0']

abbrMonths=[u'\u0a9c\u0abe\u0aa8\u0acd\u0aaf\u0ac1', u'\u0aab\u0ac7\u0aac\u0acd\u0ab0\u0ac1', u'\u0aae\u0abe\u0ab0\u0acd\u0a9a', u'\u0a8f\u0aaa\u0acd\u0ab0\u0abf\u0ab2', u'\u0aae\u0ac7', u'\u0a9c\u0ac2\u0aa8', u'\u0a9c\u0ac1\u0ab2\u0abe\u0a88', u'\u0a91\u0a97\u0ab8\u0acd\u0a9f', u'\u0ab8\u0aaa\u0acd\u0a9f\u0ac7', u'\u0a91\u0a95\u0acd\u0a9f\u0acb', u'\u0aa8\u0ab5\u0ac7', u'\u0aa1\u0abf\u0ab8\u0ac7']

days=[u'\u0ab8\u0acb\u0aae\u0ab5\u0abe\u0ab0', u'\u0aae\u0a82\u0a97\u0ab3\u0ab5\u0abe\u0ab0', u'\u0aac\u0ac1\u0aa7\u0ab5\u0abe\u0ab0', u'\u0a97\u0ac1\u0ab0\u0ac1\u0ab5\u0abe\u0ab0', u'\u0ab6\u0ac1\u0a95\u0acd\u0ab0\u0ab5\u0abe\u0ab0', u'\u0ab6\u0aa8\u0abf\u0ab5\u0abe\u0ab0', u'\u0ab0\u0ab5\u0abf\u0ab5\u0abe\u0ab0']

abbrDays=[u'\u0ab8\u0acb\u0aae', u'\u0aae\u0a82\u0a97\u0ab3', u'\u0aac\u0ac1\u0aa7', u'\u0a97\u0ac1\u0ab0\u0ac1', u'\u0ab6\u0ac1\u0a95\u0acd\u0ab0', u'\u0ab6\u0aa8\u0abf', u'\u0ab0\u0ab5\u0abf']

dateFormats={'medium': '%Y %%(abbrmonthname)s %d ', 'full': '%%(dayname)s, %Y %%(monthname)s %d ', 'long': '%Y %%(monthname)s %d ', 'short': '%y/%m/%d'}

numericSymbols={'group': ',', 'nativeZeroDigit': u'\u0ae6', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
