# Formatting configuration for locale kw

languages={'kw': 'kernewek'}

countries={'GB': 'Rywvaneth Unys'}

months=['Mys Genver', 'Mys Whevrel', 'Mys Merth', 'Mys Ebrel', 'Mys Me', 'Mys Efan', 'Mys Gortheren', 'Mye Est', 'Mys Gwyngala', 'Mys Hedra', 'Mys Du', 'Mys Kevardhu']

abbrMonths=['Gen', 'Whe', 'Mer', 'Ebr', 'Me', 'Efn', 'Gor', 'Est', 'Gwn', 'Hed', 'Du', 'Kev']

days=['De Lun', 'De Merth', 'De Merher', 'De Yow', 'De Gwener', 'De Sadorn', 'De Sul']

abbrDays=['Lun', 'Mth', 'Mhr', 'Yow', 'Gwe', 'Sad', 'Sul']

dateFormats={'medium': '%Y %%(abbrmonthname)s %d ', 'full': '%%(dayname)s, %Y %%(monthname)s %d ', 'long': '%Y %%(monthname)s %d ', 'short': '%y/%m/%d'}

numericSymbols={'group': ',', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
