# Formatting configuration for locale vi_VN

languages={'ru': u'Ti\u1ebfng Nga', 'eo': u'Ti\u1ebfng Esp\xe9ranto', 'en': u'Ti\u1ebfng Anh', 'nl': u'Ti\u1ebfng H\xe0 Lan', 'ro': u'Ti\u1ebfng La M\xe3', 'pt': u'Ti\u1ebfng B\u1ed3 \u0110\xe0o Nha', 'hr': u'Ti\u1ebfng Cr\xf4atia', 'tr': u'Ti\u1ebfng Th\u1ed5 Nh\u0129 K\u1ef3', 'ca': u'Ti\u1ebfng Catalan', 'de': u'Ti\u1ebfng \u0110\u1ee9c', 'ko': u'Ti\u1ebfng H\xe0n Qu\u1ed1c', 'fr': u'Ti\u1ebfng Ph\xe1p', 'sv': u'Ti\u1ebfng Thu\u1ef5 \u0110i\u1ec3n', 'da': u'Ti\u1ebfng \u0110an M\u1ea1ch', 'vi': u'Ti\u1ebfng Vi\u1ec7t', 'it': u'Ti\u1ebfng \xdd', 'zh': u'Ti\u1ebfng Trung Qu\u1ed1c', 'hu': u'Ti\u1ebfng Hung-ga-ri', 'ja': u'Ti\u1ebfng Nh\u1eadt', 'es': u'Ti\u1ebfng T\xe2y Ban Nha'}

countries={'BD': u'B\u0103ng-la-\u0111\xe9t', 'BE': u'B\u1ec9', 'BF': u'Bu\u1ed1c-ki-na Pha-x\xf4', 'BG': 'Bun-ga-ri', 'BA': u'B\xf4-xni-a H\xe9c-x\xea-g\xf4-vi-na', 'BB': u'B\xe1c-ba-\u0111\u1ed1t', 'BN': u'Bru-n\xe2y', 'BO': u'B\xf4-li-vi-a', 'BH': 'Ba-ren', 'BI': u'Bu-run-\u0111i', 'BJ': u'B\xea-nanh', 'JM': 'Ha-mai-ca', 'BW': u'B\u1ed1t-xoa-na', 'WS': 'Xa-moa', 'BR': 'Bra-xin', 'BS': 'Ba-ha-ma', 'BY': u'B\xea-la-r\xfat', 'BZ': u'B\xea-li-x\xea', 'RU': 'Nga', 'RW': u'Ru-an-\u0111a', 'TM': u'Tu\u1ed1c-m\xea-ni-xtan', 'TJ': u'T\xe1t-gi-ki-xtan', 'RO': 'Ru-ma-ni', 'GW': u'Ghi-n\xea B\xedt-xao', 'GT': u'Goa-t\xea-ma-la', 'GR': u'Hy L\u1ea1p', 'GQ': u'Ghi-n\xea X\xedch-\u0111\u1ea1o', 'JP': u'Nh\u1eadt B\u1ea3n', 'GY': 'Guy-a-na', 'GE': 'Gru-di-a', 'GD': u'Gr\xea-na-\u0111a', 'GB': u'V\u01b0\u01a1ng qu\u1ed1c Anh', 'GA': u'Ga-b\xf4ng', 'SV': u'En-san-va-\u0111o', 'GN': u'Ghi-n\xea', 'GM': u'G\u0103m-bi-a', 'GH': 'Gha-na', 'OM': u'\xd4-man', 'TN': 'Tuy-ni-di', 'JO': u'Gi\xf3c-\u0111a-ni', 'SP': u'S\xe9cbia', 'HR': u'Cr\xf4-a-ti-a', 'HT': 'Ha-i-ti', 'HU': 'Hung-ga-ri', 'HN': u'H\xf4n-\u0111u-r\xe1t', 'PT': u'B\u1ed3 \xd0\xe0o Nha', 'PY': 'Pa-ra-goay', 'PA': 'Pa-na-ma', 'PG': u'Pa-pu-a Niu Ghi-n\xea', 'PE': u'P\xea-ru', 'PK': 'Pa-ki-xtan', 'PH': 'Phi-lip-pin', 'PL': 'Ba Lan', 'ZM': u'D\u0103m-bi-a', 'EH': u'Ph\xeda t\xe2y Sahara', 'EE': u'E-xt\xf4-ni-a', 'EG': u'Ai-c\u1eadp', 'ZA': 'Nam Phi', 'EC': u'\xca-cu-a-\u0111o', 'IT': u'\xdd', 'VN': u'Vi\u1ec7t Nam', 'SB': u'Qu\u1ea7n \u0111\u1ea3o X\xf4-l\xf4-m\xf4ng', 'ET': u'\xca-ti-\xf4-pi-a', 'SO': u'X\xf4-ma-li', 'ZW': u'Dim-ba-bu-\xea', 'ES': u'T\xe2y Ban Nha', 'ER': u'\xca-ri-t\u01a1-r\xea-a', 'MD': u'M\xf4n-\u0111\xf4-va', 'MG': u'Ma-\u0111a-g\xe1t-xca', 'MA': u'Ma-r\u1ed1c', 'MC': u'M\xf4-na-c\xf4', 'UZ': u'U-d\u01a1-b\xea-ki-xtan', 'MM': 'Mi-an-ma', 'ML': 'Ma-li', 'MN': u'M\xf4ng C\u1ed5', 'MH': u'Qu\u1ea7n \u0111\u1ea3o M\xe1c-san', 'MK': u'Ma-x\xea-\u0111\xf4-ni-a', 'MU': u'M\xf4-ri-x\u01a1', 'MT': 'Man-ta', 'MW': 'Ma-la-uy', 'MV': u'Man-\u0111i-v\u01a1', 'MR': u'M\xf4-ri-ta-ni', 'UG': u'U-gan-\u0111a', 'MY': 'Ma-lay-xi-a', 'MX': u'M\xea-hi-c\xf4', 'IL': 'I-xra-en', 'FR': u'Ph\xe1p', 'FI': u'Ph\u1ea7n Lan', 'FJ': 'Phi-gi', 'FM': u'Mi-cr\xf4-n\xea-xi-a', 'NI': 'Ni-ca-ra-goa', 'NL': u'H\xe0 Lan', 'NO': 'Na Uy', 'NA': 'Nam-mi-bi-a', 'VU': 'Va-nu-a-tu', 'NE': u'Ni-gi\xea', 'NG': u'Ni-gi\xea-ri-a', 'NZ': u'Niu Di-l\xe2n', 'NP': u'N\xea-pan', 'CI': u'B\u1edd Bi\u1ec3n Ng\xe0', 'CH': u'Thu\u1ef5 S\u0129', 'CO': u'C\xf4-l\xf4m-bi-a', 'CN': u'Trung Qu\u1ed1c', 'CM': u'Ca-m\u01a1-run', 'CL': u'Chi-l\xea', 'CA': u'Ca-na-\u0111a', 'CG': u'C\xf4ng-g\xf4', 'CF': u'C\u1ed9ng h\xf2a Trung Phi', 'CZ': u'C\u1ed9ng h\xf2a S\xe9c', 'CY': u'S\xedp', 'CR': u'C\u1ed1t-xta Ri-ca', 'CV': u'C\xe1p-ve', 'CU': 'Cu-ba', 'SZ': 'Xoa-di-len', 'SY': 'Xi-ri', 'KG': u'C\u01b0-r\u01a1-g\u01b0-xtan', 'KE': u'K\xea-ni-a', 'SR': 'Xu-ri-nam', 'KI': 'Ki-ri-ba-ti', 'KH': 'Cam-pu-chia', 'KN': u'Xan-k\xedt v\xe0 N\xea-vi', 'KM': u'C\xf4-m\xf4', 'ST': u'Xao T\xf4-m\xea v\xe0 Prin-xi-p\xea', 'SK': u'Xl\xf4-va-ki-a', 'KR': u'H\xe0n Qu\u1ed1c', 'SI': u'Xl\xf4-ven-ni-a', 'KP': u'Tri\u1ec1u Ti\xean', 'KW': u'C\xf4-o\xe9t', 'SN': u'X\xea-n\xea-gan', 'SM': u'Xan Ma-ri-n\xf4', 'SL': u'Xi-\xea-ra L\xea-\xf4n', 'SC': u'X\xe2y-sen', 'KZ': u'Ka-d\u1eafc-xtan', 'SA': u'\u1ea2-r\u1eadp X\xea-\xfat', 'SG': 'Xin-ga-po', 'SE': u'Thu\u1ef5 \u0110i\u1ec3', 'SD': u'Xu-\u0111\u0103ng', 'DJ': 'Gi-bu-ti', 'DK': u'\u0110an M\u1ea1c', 'DE': u'\xd0\u1ee9c', 'YE': u'Y-\xea-men', 'DZ': u'An-gi\xea-ri', 'US': u'H\u1ee3p ch\xfang qu\u1ed1c Hoa k\u1ef3', 'UY': 'U-ru-goay', 'LB': u'Li-b\u0103ng', 'LC': 'Xan Lu-xi', 'LA': u'L\xe0o', 'TV': 'Tu-va-lu', 'TW': u'\xd0\xe0i Loan', 'TT': u'Tri-ni-\u0111\xe1t v\xe0 T\xf4-ba-g\xf4', 'TR': u'Th\u1ed5 Nh\u0129 K\u1ef3', 'LK': 'Xri Lan-ca', 'LI': u'Lich-ten-xt\xean', 'LV': u'L\xe1t-vi-a', 'TO': u'T\xf4ng-ga', 'LT': 'Li-tu-a-ni-a', 'LU': u'L\xfac-x\u0103m-bua', 'LR': u'Li-b\xea-ri-a', 'LS': u'L\xea-x\xf4-th\xf4', 'TH': u'Th\xe1i Lan', 'TG': u'T\xf4-g\xf4', 'TD': u'S\xe1t', 'LY': 'Li-bi', 'VA': u'Va-ti-c\u0103ng', 'VC': u'Xan Vin-xen v\xe0 Gr\xea-na-din', 'AE': u'C\xe1c Ti\u1ec3u V\u01b0\u01a1ng qu\u1ed1c A-r\u1eadp Th\u1ed1ng nh\u1ea5t', 'VE': u'V\xea-n\xea-zu-\xea-la', 'AG': u'An-ti-gu-a v\xe0 Ba-bu-\u0111a', 'AF': u'\xc1p-ga-ni-xtan', 'IQ': u'I-r\u1eafc', 'IS': u'Ai-x\u01a1-len', 'IR': 'I-ran', 'AM': u'\xc1c-m\xea-ni-a', 'AL': 'An-ba-ni', 'AO': u'\u0102ng-g\xf4-la', 'AR': u'\xc1c-hen-ti-na', 'AU': u'\xdac \xd0\u1ea1i L\u1ee3i', 'AT': u'\xc1o', 'IN': u'\u1ea4n \xd0\u1ed9', 'TZ': 'Tan-da-ni-a', 'AZ': u'Ai-d\xe9c-bai-gian', 'IE': 'Ai-len', 'ID': u'Nam D\u01b0\u01a1ng', 'UA': 'U-crai-na', 'QA': 'Ca-ta', 'MZ': u'M\xf4-d\u0103m-b\xedch'}

months=[u'Th\xe1ng m\u1ed9t', u'Th\xe1ng hai', u'Th\xe1ng ba', u'Th\xe1ng t\u01b0', u'Th\xe1ng n\u0103m', u'Th\xe1ng s\xe1u', u'Th\xe1ng b\u1ea3y', u'Th\xe1ng t\xe1m', u'Th\xe1ng ch\xedn', u'Th\xe1ng m\u01b0\u1eddi', u'Th\xe1ng m\u01b0\u1eddi m\u1ed9t', u'Th\xe1ng m\u01b0\u1eddi hai']

abbrMonths=['Thg 1', 'Thg 2', 'Thg 3', 'Thg 4', 'Thg 5', 'Thg 6', 'Thg 7', 'Thg 8', 'Thg 9', 'Thg 10', 'Thg 11', 'Thg 12']

days=[u'Th\u1ee9 hai', u'Th\u1ee9 ba', u'Th\u1ee9 t\u01b0', u'Th\u1ee9 n\u0103m', u'Th\u1ee9 s\xe1u', u'Th\u1ee9 b\u1ea3y', u'Ch\u1ee7 nh\u1eadt']

abbrDays=['Th 2', 'Th 3', 'Th 4', 'Th 5', 'Th 6', 'Th 7', 'CN']

dateFormats={'medium': '%d-%m-%Y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%Y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
