# Formatting configuration for locale kn_IN

languages={'kn': u'\u0c95\u0ca8\u0ccd\u0ca8\u0ca1'}

countries={'IN': u'\u0cad\u0cbe\u0cb0\u0ca4'}

months=[u'\u0c9c\u0ca8\u0cb5\u0cb0\u0cc0', u'\u0cab\u0cc6\u0cac\u0ccd\u0cb0\u0cb5\u0cb0\u0cc0', u'\u0cae\u0cbe\u0cb0\u0ccd\u0c9a\u0ccd', u'\u0c8e\u0caa\u0ccd\u0cb0\u0cbf\u0cb2\u0ccd', u'\u0cae\u0cc6', u'\u0c9c\u0cc2\u0ca8\u0ccd', u'\u0c9c\u0cc1\u0cb2\u0cc8', u'\u0c86\u0c97\u0cb8\u0ccd\u0c9f\u0ccd', u'\u0cb8\u0caa\u0ccd\u0c9f\u0cc6\u0c82\u0cac\u0cb0\u0ccd', u'\u0c85\u0c95\u0ccd\u0c9f\u0ccb\u0cac\u0cb0\u0ccd', u'\u0ca8\u0cb5\u0cc6\u0c82\u0cac\u0cb0\u0ccd', u'\u0ca1\u0cbf\u0cb8\u0cc6\u0c82\u0cac\u0cb0\u0ccd']

abbrMonths=[u'\u0c9c\u0ca8\u0cb5\u0cb0\u0cc0', u'\u0cab\u0cc6\u0cac\u0ccd\u0cb0\u0cb5\u0cb0\u0cc0', u'\u0cae\u0cbe\u0cb0\u0ccd\u0c9a\u0ccd', u'\u0c8e\u0caa\u0ccd\u0cb0\u0cbf\u0cb2\u0ccd', u'\u0cae\u0cc6', u'\u0c9c\u0cc2\u0ca8\u0ccd', u'\u0c9c\u0cc1\u0cb2\u0cc8', u'\u0c86\u0c97\u0cb8\u0ccd\u0c9f\u0ccd', u'\u0cb8\u0caa\u0ccd\u0c9f\u0cc6\u0c82\u0cac\u0cb0\u0ccd', u'\u0c85\u0c95\u0ccd\u0c9f\u0ccb\u0cac\u0cb0\u0ccd', u'\u0ca8\u0cb5\u0cc6\u0c82\u0cac\u0cb0\u0ccd', u'\u0ca1\u0cbf\u0cb8\u0cc6\u0c82\u0cac\u0cb0\u0ccd']

days=[u'\u0cb8\u0ccb\u0cae\u0cb5\u0cbe\u0cb0', u'\u0cae\u0c82\u0c97\u0cb3\u0cb5\u0cbe\u0cb0', u'\u0cac\u0cc1\u0ca7\u0cb5\u0cbe\u0cb0', u'\u0c97\u0cc1\u0cb0\u0cc1\u0cb5\u0cbe\u0cb0', u'\u0cb6\u0cc1\u0c95\u0ccd\u0cb0\u0cb5\u0cbe\u0cb0', u'\u0cb6\u0ca8\u0cbf\u0cb5\u0cbe\u0cb0', u'\u0cb0\u0cb5\u0cbf\u0cb5\u0cbe\u0cb0']

abbrDays=[u'\u0cb8\u0ccb.', u'\u0cae\u0c82.', u'\u0cac\u0cc1.', u'\u0c97\u0cc1.', u'\u0cb6\u0cc1.', u'\u0cb6\u0ca8\u0cbf.', u'\u0cb0.']

dateFormats={'medium': '%d-%m-%Y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d-%m-%y'}

numericSymbols={'group': ',', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': '.', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
