# Formatting configuration for locale it

languages={'vi': 'vietnamita', 'el': 'greco', 'eo': 'esperanto', 'en': 'inglese', 'zh': 'cinese', 'af': 'afrikaans', 'sw': 'swahili', 'ca': 'catalano', 'am': 'amarico', 'gu': 'gujarati', 'sv': 'svedese', 'cs': 'ceco', 'ar': 'arabo', 'ga': 'irlandese', 'eu': 'basco', 'et': 'estone', 'gl': 'galiziano', 'id': 'indonesiano', 'es': 'spagnolo', 'ru': 'russo', 'nl': 'olandese', 'nn': 'norvegese nynorsk', 'no': 'norvegese', 'nb': u'norvegese bokm\xe5l', 'tr': 'turco', 'lv': 'lettone', 'lt': 'lituano', 'th': 'thai', 'ti': 'tigrinya', 'it': 'italiano', 'so': 'somalo', 'he': 'ebreo', 'is': 'islandese', 'pl': 'polacco', 'haw': 'hawaiano', 'be': 'bielorusso', 'fr': 'francese', 'bg': 'bulgaro', 'sl': 'sloveno', 'hr': 'croato', 'iu': 'inuktitut', 'bn': 'bengali', 'de': 'tedesco', 'da': 'danese', 'fa': 'persiano', 'hi': 'hindi', 'fi': 'finlandese', 'hy': 'armeno', 'hu': 'ungherese', 'ja': 'giapponese', 'fo': 'faroese', 'om': 'oromo', 'ro': 'romeno', 'gv': 'manx', 'pt': 'portoghese', 'sr': 'serbo', 'sq': 'albanian', 'ko': 'coreano', 'kn': 'kannada', 'mk': 'macedone', 'kl': 'kalaallisut', 'sk': 'slovacco', 'mt': 'maltese', 'sh': 'serbo-croata', 'kw': 'cornish', 'uk': 'ucraino', 'mr': 'marathi', 'ta': 'tamil'}

countries={'BD': 'Bangladesh', 'BE': 'Belgio', 'BF': 'Burkina Faso', 'BG': 'Bulgaria', 'BA': 'Bosnia Erzegovina', 'BB': 'Barbados', 'WF': 'Wallis e Futuna', 'BM': 'Bermuda', 'BN': 'Brunei', 'BO': 'Bolivia', 'BH': 'Bahrain', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Giamaica', 'BV': 'Isola Bouvet', 'BW': 'Botswana', 'WS': 'Samoa', 'BR': 'Brasile', 'BS': 'Bahamas', 'BY': 'Bielorussia', 'BZ': 'Belize', 'RU': 'Federazione Russa', 'RW': 'Ruanda', 'TL': 'Timor Est', 'RE': 'Reunion', 'TM': 'Turkmenistan', 'TJ': 'Tajikistan', 'RO': 'Romania', 'TK': 'Tokelau', 'GW': 'Guinea-Bissau', 'GU': 'Guam', 'GT': 'Guatemala', 'GS': 'Sud Georgia e Isole Sandwich del Sud', 'GR': 'Grecia', 'GQ': 'Guinea Equatoriale', 'GP': 'Guadalupa', 'JP': 'Giappone', 'GY': 'Guyana', 'GF': 'Guiana Francese', 'GE': 'Georgia', 'GD': 'Grenada', 'GB': 'Regno Unito', 'GA': 'Gabon', 'SV': 'El Salvador', 'GN': 'Guinea', 'GM': 'Gambia', 'GL': 'Groenlandia', 'GI': 'Gibilterra', 'GH': 'Ghana', 'OM': 'Oman', 'TN': 'Tunisia', 'JO': 'Giordania', 'SP': 'Serbia', 'HR': 'Croazia', 'HT': 'Haiti', 'HU': 'Ungheria', 'HK': 'Hong Kong (Cina)', 'HN': 'Honduras', 'HM': 'Isole Heard ed Isole McDonald', 'VE': 'Venezuela', 'PR': 'Portorico', 'PS': 'Palestina', 'PW': 'Palau', 'PT': 'Portogallo', 'SJ': 'Svalbard e Jan Mayen', 'PY': 'Paraguay', 'IQ': 'Iraq', 'PA': 'Panama', 'PF': 'Polinesia Francese', 'PG': 'Papua Nuova Guinea', 'PE': 'Peru', 'PK': 'Pakistan', 'PH': 'Filippine', 'PN': 'Pitcairn', 'PL': 'Polonia', 'PM': 'Saint Pierre e Miquelon', 'ZM': 'Zambia', 'EH': 'Sahara Occidentale', 'EE': 'Estonia', 'EG': 'Egitto', 'ZA': 'Sud Africa', 'EC': 'Ecuador', 'IT': 'Italia', 'VN': 'Vietnam', 'SB': 'Isole Solomon', 'ET': 'Etiopia', 'SO': 'Somalia', 'ZW': 'Zimbabwe', 'SA': 'Arabia Saudita', 'ES': 'Spagna', 'ER': 'Eritrea', 'MD': 'Moldavia', 'MG': 'Madagascar', 'MA': 'Marocco', 'MC': 'Monaco', 'UZ': 'Uzbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MO': 'Macao (Cina)', 'MN': 'Mongolia', 'MH': 'Isole Marshall', 'MK': 'Macedonia, Repubblica', 'MU': 'Mauritius', 'MT': 'Malta', 'MW': 'Malawi', 'MV': 'Maldive', 'MQ': 'Martinica', 'MP': 'Northern Mariana Islands', 'MS': 'Montserrat', 'MR': 'Mauritania', 'UG': 'Uganda', 'MY': 'Malesia', 'MX': 'Messico', 'IL': 'Israele', 'FR': 'Francia', 'IO': "Territori Britannici dell'Oceano Indiano", 'SH': 'Saint Helena', 'FI': 'Finlandia', 'FJ': 'Fiji', 'FK': 'Isole Falkland', 'FM': 'Micronesia', 'FO': 'Isole Faroe', 'NI': 'Nicaragua', 'NL': 'Olanda', 'NO': 'Norvegia', 'NA': 'Namibia', 'VU': 'Vanuatu', 'NC': 'Nuova Caledonia', 'NE': 'Niger', 'NF': 'Isola Norfolk', 'NG': 'Nigeria', 'NZ': 'Nuova Zelanda', 'NP': 'Nepal', 'NR': 'Nauru', 'NU': 'Niue', 'CK': 'Isole Cook', 'CI': "Costa d'Avorio", 'CH': 'Svizzera', 'CO': 'Colombia', 'CN': 'Cina', 'CM': 'Camerun', 'CL': 'Cile', 'CC': 'Isole Cocos (Keeling)', 'CA': 'Canada', 'CG': 'Congo', 'CF': 'Repubblica Centrafricana', 'CD': 'Repubblica Democratica del Congo', 'CZ': 'Repubblica Ceca', 'CY': 'Cipro', 'CX': 'Isola di Natale', 'CR': 'Costa Rica', 'Fallback': 'en', 'CV': 'Capo Verde', 'CU': 'Cuba', 'SZ': 'Swaziland', 'SY': 'Siria', 'KG': 'Kirghizistan', 'KE': 'Kenya', 'SR': 'Suriname', 'KI': 'Kiribati', 'KH': 'Cambogia', 'KN': 'Saint Kitts e Nevis', 'KM': 'Comore', 'ST': 'Sao Tome e Principe', 'SK': 'Slovacchia', 'KR': 'Corea del Sud', 'SI': 'Slovenia', 'KP': 'Corea del Nord', 'KW': 'Kuwait', 'SN': 'Senegal', 'SM': 'San Marino', 'SL': 'Sierra Leone', 'SC': 'Seychelles', 'KZ': 'Kazakistan', 'KY': 'Isole Cayman', 'SG': 'Singapore', 'SE': 'Svezia', 'SD': 'Sudan', 'DO': 'Repubblica Dominicana', 'DM': 'Dominica', 'DJ': 'Gibuti', 'DK': 'Danimarca', 'VG': 'Isole Vergini Britanniche', 'DE': 'Germania', 'YE': 'Yemen', 'DZ': 'Algeria', 'US': 'Stati Uniti', 'UY': 'Uruguay', 'YU': 'Yugoslavia', 'YT': 'Mayotte', 'UM': 'Isole minori Americane', 'LB': 'Libano', 'LC': 'Santa Lucia', 'LA': 'Laos', 'TV': 'Tuvalu', 'TW': 'Taiwan (provincia della Cina)', 'TT': 'Trinidad e Tobago', 'TR': 'Turchia', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': 'Lettonia', 'TO': 'Tonga', 'LT': 'Lituania', 'LU': 'Lussemburgo', 'LR': 'Liberia', 'LS': 'Lesotho', 'TH': 'Thailandia', 'TF': 'Territori francesi del sud', 'TG': 'Togo', 'TD': 'Ciad', 'TC': 'Isole Turks e Caicos', 'LY': 'Libia', 'VA': 'Vaticano', 'VC': 'Saint Vincent e Grenadines', 'AE': 'Emirati Arabi Uniti', 'AD': 'Andorra', 'AG': 'Antigua e Barbuda', 'AF': 'Afghanistan', 'AI': 'Anguilla', 'VI': 'Isole Vergini Americane', 'IS': 'Islanda', 'IR': 'Iran', 'AM': 'Armenia', 'AL': 'Albania', 'AO': 'Angola', 'AN': 'Antille Olandesi', 'AQ': 'Antartico', 'AS': 'Samoa Americane', 'AR': 'Argentina', 'AU': 'Australia', 'AT': 'Austria', 'AW': 'Aruba', 'IN': 'India', 'TZ': 'Tanzania', 'AZ': 'Azerbaijan', 'IE': 'Irlanda', 'ID': 'Indonesia', 'UA': 'Ucraina', 'QA': 'Qatar', 'MZ': 'Mozambico'}

months=['gennaio', 'febbraio', 'marzo', 'aprile', 'maggio', 'giugno', 'luglio', 'agosto', 'settembre', 'ottobre', 'novembre', 'dicembre']

abbrMonths=['gen', 'feb', 'mar', 'apr', 'mag', 'giu', 'lug', 'ago', 'set', 'ott', 'nov', 'dic']

days=[u'luned\xec', u'marted\xec', u'mercoled\xec', u'gioved\xec', u'venerd\xec', 'sabato', 'domenica']

abbrDays=['lun', 'mar', 'mer', 'gio', 'ven', 'sab', 'dom']

dateFormats={'medium': '%d/%%(abbrmonthname)s/%y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d/%m/%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
