# Formatting configuration for locale fi

languages={'vi': 'vietnam', 'el': 'kreikka', 'eo': 'esperanto', 'en': 'englanti', 'zh': 'kiina', 'af': 'afrikanhollanti', 'sw': 'suahili', 'ca': 'katalaani', 'am': 'amharan kieli', 'gu': 'gujarati', 'sv': 'ruotsi', 'cs': 'tsekki', 'ar': 'arabia', 'mk': 'makedonia', 'ga': 'iiri', 'eu': 'baski', 'et': 'viro', 'az': 'azerbaizani', 'id': 'indonesia', 'es': 'espanja', 'ru': u'ven\xe4j\xe4', 'gl': 'galicia', 'nl': 'hollanti', 'nn': 'uusnorja', 'la': 'latina', 'nb': 'kirjanorja', 'tr': 'turkki', 'lv': 'latvia', 'lt': 'liettua', 'tk': 'tagalog', 'th': 'thai', 'ti': 'tigrinja', 'it': 'italia', 'so': 'somali', 'he': 'heprea', 'is': 'islanti', 'pl': 'puola', 'haw': 'havaiji', 'be': u'valkoven\xe4j\xe4', 'fr': 'ranska', 'bg': 'bulgaria', 'uk': 'ukraina', 'sl': 'sloveeni', 'hr': 'kroaatti', 'iu': 'eskimo', 'bn': 'bengali', 'de': 'saksa', 'bh': 'bihari', 'om': 'oromo', 'da': 'tanska', 'fa': 'farsi', 'hi': 'hindi', 'fi': 'suomi', 'hy': 'armenia', 'hu': 'unkari', 'ur': 'urdu', 'ja': 'japani', 'fo': u'f\xe4\xe4ri', 'ka': 'georgia', 'ro': 'romania', 'gv': 'manx', 'pt': 'portugali', 'uz': 'uzbekki', 'kk': 'kazakki', 'sr': 'serbia', 'sq': 'albania', 'no': 'norja', 'ko': 'korea', 'kn': 'kannada', 'km': 'khmer', 'kl': u'gr\xf6\xf6nlandi', 'sk': 'slovakki', 'mt': 'malta', 'sh': 'serbokroatia', 'kw': 'korni', 'ku': 'kurdi', 'mr': 'marathi', 'te': 'telugu', 'ta': 'tamili', 'my': 'burma'}

countries={'BD': 'Bangladesh', 'BE': 'Belgia', 'BF': 'Burkina Faso', 'BG': 'Bulgaria', 'BA': 'Bosnia ja Hertsegovina', 'BB': 'Barbados', 'WF': 'Wallis ja Futuna', 'BM': 'Bermuda', 'BN': 'Brunei', 'BO': 'Bolivia', 'BH': 'Bahrain', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaica', 'BV': "Bouvet'nsaari", 'BW': 'Botswana', 'WS': 'Samoa', 'BR': 'Brasilia', 'BS': 'Bahama', 'BY': u'Valko-Ven\xe4j\xe4', 'BZ': 'Belize', 'RU': u'Ven\xe4j\xe4', 'RW': 'Ruanda', 'TL': u'It\xe4-Timor', 'RE': u'R\xe9union', 'TM': 'Turkmenistan', 'TJ': 'Tadzikistan', 'RO': 'Romania', 'TK': 'Tokelau', 'GW': 'Guinea-Bissau', 'GU': 'Guam', 'GT': 'Guatemala', 'GS': u'Etel\xe4-Georgia ja Etel\xe4iset Sandwichsaaret', 'GR': 'Kreikka', 'GQ': u'P\xe4iv\xe4ntasaajan Guinea', 'GP': 'Guadeloupe', 'JP': 'Japani', 'GY': 'Guyana', 'GF': 'Ranskan Guayana', 'GE': 'Georgia', 'GD': 'Grenada', 'GB': 'Iso-Britannia', 'GA': 'Gabon', 'SV': 'El Salvador', 'GN': 'Guinea', 'GM': 'Gambia', 'GL': u'Gr\xf6nlanti', 'GI': 'Gibraltar', 'GH': 'Ghana', 'OM': 'Oman', 'TN': 'Tunisia', 'JO': 'Jordania', 'SP': 'Serbia', 'HR': 'Kroatia', 'HT': 'Haiti', 'HU': 'Unkari', 'HK': 'Hongkongin erityishallintoalue', 'HN': 'Honduras', 'HM': 'Heard ja McDonaldinsaaret', 'VE': 'Venezuela', 'PR': 'Puerto Rico', 'PS': 'Palestiina', 'PW': 'Palau', 'PT': 'Portugali', 'SJ': 'Huippuvuoret ja Jan Mayen', 'PY': 'Paraguay', 'IQ': 'Irak', 'PA': 'Panama', 'PF': 'Ranskan Polynesia', 'PG': 'Papua-Uusi-Guinea', 'PE': 'Peru', 'PK': 'Pakistan', 'PH': 'Filippiinit', 'PN': 'Pitcairn', 'PL': 'Puola', 'PM': 'Saint-Pierre ja Miquelon', 'ZM': 'Sambia', 'EH': u'L\xe4nsi-Sahara', 'EE': 'Viro', 'EG': 'Egypti', 'ZA': u'Etel\xe4-Afrikka', 'EC': 'Equador', 'IT': 'Italia', 'VN': 'Vietnam', 'SB': 'Salomonsaaret', 'ET': 'Etiopia', 'SO': 'Somalia', 'ZW': 'Zimbabwe', 'SA': 'Saudi-Arabia', 'ES': 'Espanja', 'ER': 'Eritrea', 'MD': 'Moldova', 'MG': 'Madagaskar', 'MA': 'Marokko', 'MC': 'Monaco', 'UZ': 'Uzbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MO': 'Macaon erityishallintoalue', 'MN': 'Mongolia', 'MH': 'Marshallinsaaret', 'MK': 'Makedonia', 'MU': 'Mauritius', 'MT': 'Malta', 'MW': 'Malawi', 'MV': 'Malediivit', 'MQ': 'Martinique', 'MP': 'Pohjois-Mariaanit', 'MS': 'Montserrat', 'MR': 'Mauritania', 'UG': 'Uganda', 'MY': 'Malesia', 'MX': 'Meksiko', 'IL': 'Israel', 'FR': 'Ranska', 'IO': u'Brittil\xe4inen Intian valtameren alue', 'SH': 'Saint Helena', 'FI': 'Suomi', 'FJ': 'Fidzi', 'FK': 'Falklandinsaaret', 'FM': 'Mikronesia', 'FO': u'F\xe4rsaaret', 'NI': 'Nicaragua', 'NL': 'Alankomaat', 'NO': 'Norja', 'NA': 'Namibia', 'VU': 'Vanuatu', 'NC': 'Uusi-Kaledonia', 'NE': 'Niger', 'NF': 'Norfolkinsaari', 'NG': 'Nigeria', 'NZ': 'Uusi Seelanti', 'NP': 'Nepal', 'NR': 'Nauru', 'NU': 'Niue', 'CK': 'Cookinsaaret', 'CI': 'Norsunluurannikko', 'CH': 'Sveitsi', 'CO': 'Kolumbia', 'CN': 'Kiina', 'CM': 'Kamerun', 'CL': 'Chile', 'CC': 'Kookossaaret', 'CA': 'Kanada', 'CG': 'Kongo', 'CF': 'Keski-Afrikan tasavalta', 'CD': 'Kongon demokraattinen tasavalta', 'CZ': 'Tsekin tasavalta', 'CY': 'Kypros', 'CX': 'Joulusaari', 'CR': 'Costa Rica', 'Fallback': 'en', 'CV': 'Kap Verde', 'CU': 'Kuuba', 'SZ': 'Swazimaa', 'SY': 'Syyria', 'KG': 'Kirgisia', 'KE': 'Kenia', 'SR': 'Suriname', 'KI': 'Kiribati', 'KH': 'Kambodza', 'KN': 'Saint Kitts ja Nevis', 'KM': 'Komorit', 'ST': 'Sao Tome ja Principe', 'SK': 'Slovakia', 'KR': u'Etel\xe4-Korea', 'SI': 'Slovenia', 'KP': 'Pohjois-Korea', 'KW': 'Kuwait', 'SN': 'Senegal', 'SM': 'San Marino', 'SL': 'Sierra Leone', 'SC': 'Seychellit', 'KZ': 'Kazakstan', 'KY': 'Caymansaaret', 'SG': 'Singapore', 'SE': 'Ruotsi', 'SD': 'Sudan', 'DO': 'Dominikaaninen tasavalta', 'DM': 'Dominica', 'DJ': 'Djibouti', 'DK': 'Tanska', 'VG': u'Brittil\xe4iset Neitsytsaaret', 'DE': 'Saksa', 'YE': 'Jemen', 'DZ': 'Algeria', 'US': 'Yhdysvallat', 'UY': 'Uruguay', 'YU': 'Jugoslavia', 'YT': 'Mayotte', 'UM': 'Yhdysvaltain pienet erillissaaret', 'LB': 'Libanon', 'LC': 'Saint Lucia', 'LA': 'Laos', 'TV': 'Tuvalu', 'TW': 'Kiinan maakunta Taiwan', 'TT': 'Trinidad ja Tobago', 'TR': 'Turkki', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': 'Latvia', 'TO': 'Tonga', 'LT': 'Liettua', 'LU': 'Luxemburg', 'LR': 'Liberia', 'LS': 'Lesotho', 'TH': 'Thaimaa', 'TF': u'Ranskan etel\xe4iset alueet', 'TG': 'Togo', 'TD': 'Tsad', 'TC': 'Turks- ja Caicossaaret', 'LY': 'Libya', 'VA': 'Vatikaani', 'VC': 'Saint Vincent ja Grenadiinit', 'AE': 'Arabiemiirikunnat', 'AD': 'Andorra', 'AG': 'Antigua ja Barbuda', 'AF': 'Afganistan', 'AI': 'Anguilla', 'VI': 'Yhdysvaltain Neitsytsaaret', 'IS': 'Islanti', 'IR': 'Iran', 'AM': 'Armenia', 'AL': 'Albania', 'AO': 'Angola', 'AN': 'Alankomaiden Antillit', 'AQ': 'Antarktis', 'AS': 'Amerikan Samoa', 'AR': 'Argentiina', 'AU': 'Australia', 'AT': u'It\xe4valta', 'AW': 'Aruba', 'IN': 'Intia', 'TZ': 'Tansania', 'AZ': 'Azerbaidzan', 'IE': 'Irlanti', 'ID': 'Indonesia', 'UA': 'Ukraina', 'QA': 'Qatar', 'MZ': 'Mosambik'}

months=['tammikuu', 'helmikuu', 'maaliskuu', 'huhtikuu', 'toukokuu', u'kes\xe4kuu', u'hein\xe4kuu', 'elokuu', 'syyskuu', 'lokakuu', 'marraskuu', 'joulukuu']

abbrMonths=['tammi', 'helmi', 'maalis', 'huhti', 'touko', u'kes\xe4', u'hein\xe4', 'elo', 'syys', 'loka', 'marras', 'joulu']

days=['maanantai', 'tiistai', 'keskiviikko', 'torstai', 'perjantai', 'lauantai', 'sunnuntai']

abbrDays=['ma', 'ti', 'ke', 'to', 'pe', 'la', 'su']

dateFormats={'medium': '%d.%m.%Y', 'full': '%d. %%(monthname)sta %Y', 'long': '%d. %%(monthname)sta %Y', 'short': '%d.%m.%Y'}

numericSymbols={'group': u'\xa0', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
