# Formatting configuration for locale fo

languages={'fo': u'f\xf8royskt'}

countries={'FO': u'F\xf8royar'}

months=['januar', 'februar', 'mars', u'apr\xedl', 'mai', 'juni', 'juli', 'august', 'september', 'oktober', 'november', 'desember']

abbrMonths=['jan', 'feb', 'mar', 'apr', 'mai', 'jun', 'jul', 'aug', 'sep', 'okt', 'nov', 'des']

days=[u'm\xe1nadagur', u't\xfdsdagur', 'mikudagur', u'h\xf3sdagur', u'fr\xedggjadagur', 'leygardagur', 'sunnudagur']

abbrDays=[u'm\xe1n', u't\xfds', 'mik', u'h\xf3s', u'fr\xed', 'ley', 'sun']

dateFormats={'medium': '%d-%m-%Y', 'full': '%%(dayname)s %d %%(monthname)s %Y', 'long': '%d. %%(abbrmonthname)s %Y', 'short': '%d-%m-%y'}

numericSymbols={'group': '.', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
