# Formatting configuration for locale et_EE

languages={'el': 'Kreeka', 'en': 'Inglise', 'zh': 'Hiina', 'it': 'Itaalia', 'ar': 'Araabia', 'cs': 'Tiehhi', 'et': 'Eesti', 'es': 'Hispaania', 'ru': 'Vene', 'nl': 'Hollandi', 'pt': 'Portugali', 'no': 'Norra', 'tr': u'T\xfcrgi', 'lv': u'L\xe4ti', 'lt': 'Leedu', 'ro': 'Rumeenia', 'pl': 'Poola', 'fr': 'Prantsuse', 'bg': 'Bulgaaria', 'hr': 'Horvaadi', 'de': 'Saksa', 'da': 'Taani', 'fi': 'Soome', 'hu': 'Ungari', 'ja': 'Jaapani', 'he': 'Heebrea', 'ko': 'Korea', 'sv': 'Rootsi', 'sk': 'Slovaki', 'sl': 'Sloveeni'}

countries={'BD': 'Bangladesh', 'BE': 'Belgia', 'BF': 'Burkina Faso', 'BG': 'Bulgaaria', 'BA': 'Bosnia ja Hertsegoviina', 'BB': 'Barbados', 'WF': 'Wallis ja Futuna', 'BM': 'Bermuda', 'BN': 'Brunei', 'BO': 'Boliivia', 'BH': 'Bahrein', 'BI': 'Burundi', 'BJ': 'Benin', 'BT': 'Bhutan', 'JM': 'Jamaica', 'BV': "Bouvet' saar", 'BW': 'Botswana', 'WS': 'Samoa', 'BR': 'Brasiilia', 'BS': 'Bahama saared', 'BY': 'Valgevene', 'BZ': 'Belize', 'RU': 'Venemaa', 'RW': 'Rwanda', 'TL': 'Ida-Timor', 'RE': u'R\xe9union', 'TM': u'T\xfcrkmenistan', 'TJ': u'Tad\u012aikistan', 'RO': 'Rumeenia', 'TK': 'Tokelau', 'GW': 'Guinea-Bissau', 'GU': 'Guam', 'GT': 'Guatemala', 'GS': u'L\xf5una-Georgia ja L\xf5una-Sandwichi saared', 'GR': 'Kreeka', 'GQ': 'Ekvatoriaal-Guinea', 'GP': 'Guadeloupe', 'JP': 'Jaapan', 'GY': 'Guyana', 'GF': 'Prantsuse Guajaana', 'GE': 'Gruusia', 'GD': 'Grenada', 'GB': u'\xdchendkuningriik', 'GA': 'Gabon', 'SV': 'El Salvador', 'GN': 'Guinea', 'GM': 'Gambia', 'GL': u'Gr\xf6\xf6nimaa', 'GI': 'Gibraltar', 'GH': 'Ghana', 'OM': 'Omaan', 'TN': 'Tuneesia', 'JO': 'Jordaania', 'SP': 'Serbia', 'HR': 'Horvaatia', 'HT': 'Haiti', 'HU': 'Ungari', 'HK': 'Hongkongi erihalduspiirkond', 'HN': 'Honduras', 'HM': 'Heard ja McDonald', 'VE': 'Venezuela', 'PR': 'Puerto Rico', 'PS': 'Palestiina Omavalitsus', 'PW': 'Belau', 'PT': 'Portugal', 'SJ': 'Svalbard ja Jan Mayen', 'PY': 'Paraguay', 'IQ': 'Iraak', 'PA': 'Panama', 'PF': u'Prantsuse Pol\xfcneesia', 'PG': 'Paapua Uus-Guinea', 'PE': 'Peruu', 'PK': 'Pakistan', 'PH': 'Filipiinid', 'PN': 'Pitcairn', 'PL': 'Poola', 'PM': 'Saint-Pierre ja Miquelon', 'ZM': 'Sambia', 'EH': u'L\xe4\xe4ne-Sahara', 'EE': 'Eesti', 'EG': 'Egiptus', 'ZA': u'L\xf5una-Aafrika Vabariik', 'EC': 'Ecuador', 'IT': 'Itaalia', 'VN': 'Vietnam', 'SB': 'Saalomoni Saared', 'ET': 'Etioopia', 'SO': 'Somaalia', 'ZW': 'Zimbabwe', 'SA': 'Saudi Araabia', 'ES': 'Hispaania', 'ER': 'Eritrea', 'MD': 'Moldova', 'MG': 'Madagaskar', 'MA': 'Maroko', 'MC': 'Monaco', 'UZ': 'Usbekistan', 'MM': 'Myanmar', 'ML': 'Mali', 'MO': 'Aomeni Hiina erihalduspiirkond', 'MN': 'Mongoolia', 'MH': 'Marshalli Saared', 'MK': 'Makedoonia Vabariik', 'MU': 'Mauritius', 'MT': 'Malta', 'MW': 'Malawi', 'MV': 'Maldiivid', 'MQ': 'Martinique', 'MP': u'P\xf5hja-Mariaanid', 'MS': 'Montserrat', 'MR': 'Mauritaania', 'UG': 'Uganda', 'MY': 'Malaisia', 'MX': 'Mehhiko', 'IL': 'Iisrael', 'FR': 'Prantsusmaa', 'IO': 'Briti India ookeani ala', 'SH': 'Saint Helena', 'FI': 'Soome', 'FJ': u'Fid\u012ai', 'FK': 'Falklandi saared', 'FM': 'Mikroneesia Liiduriigid', 'FO': u'F\xe4\xe4ri saared', 'NI': 'Nicaragua', 'NL': 'Holland', 'NO': 'Norra', 'NA': 'Namiibia', 'VU': 'Vanuatu', 'NC': 'Uus-Kaledoonia', 'NE': 'Niger', 'NF': 'Norfolk', 'NG': 'Nigeeria', 'NZ': 'Uus-Meremaa', 'NP': 'Nepal', 'NR': 'Nauru', 'NU': 'Niue', 'CK': 'Cooki saared', 'CI': "Cote d'Ivoire", 'CH': u'\u0128veits', 'CO': 'Colombia', 'CN': 'Hiina', 'CM': 'Kamerun', 'CL': 'Tiiili', 'CC': 'Kookossaared', 'CA': 'Kanada', 'CG': 'Kongo', 'CF': 'Kesk-Aafrika Vabariik', 'CD': 'Kongo DV', 'CZ': 'Tiehhi Vabariik', 'CY': u'K\xfcpros', 'CX': u'J\xf5ulusaar', 'CR': 'Costa Rica', 'Fallback': 'en', 'CV': 'Cabo Verde', 'CU': 'Kuuba', 'SZ': 'Svaasimaa', 'SY': u'S\xfc\xfcria', 'KG': u'K\xf5rg\xf5zstan', 'KE': 'Kenya', 'SR': 'Suriname', 'KI': 'Kiribati', 'KH': u'Kambod\u012aa', 'KN': 'Saint Kitts ja Nevis', 'KM': 'Komoorid', 'ST': u'Sao Tom\xe9 ja Principe', 'SK': 'Slovakkia', 'KR': u'L\xf5una-Korea', 'SI': 'Sloveenia', 'KP': u'P\xf5hja-Korea', 'KW': 'Kuveit', 'SN': 'Senegal', 'SM': 'San Marino', 'SL': 'Sierra Leone', 'SC': 'Seiiellid', 'KZ': 'Kasahstan', 'KY': 'Kaimani saared', 'SG': 'Singapur', 'SE': 'Rootsi', 'SD': 'Sudaan', 'DO': 'Dominikaani Vabariik', 'DM': 'Dominica', 'DJ': 'Djibouti', 'DK': 'Taani', 'VG': 'Briti Neitsisaared', 'DE': 'Saksamaa', 'YE': 'Jeemen', 'DZ': u'Al\u012aeeria', 'US': u'Ameerika \xdchendriigid', 'UY': 'Uruguay', 'YU': 'Jugoslaavia', 'YT': 'Mayotte', 'UM': u'\xdchendriikide hajasaared', 'LB': 'Liibanon', 'LC': 'Saint Lucia', 'LA': 'Laose DRV', 'TV': 'Tuvalu', 'TW': 'Taiwan', 'TT': 'Trinidad ja Tobago', 'TR': u'T\xfcrgi', 'LK': 'Sri Lanka', 'LI': 'Liechtenstein', 'LV': u'L\xe4ti', 'TO': 'Tonga', 'LT': 'Leedu', 'LU': 'Luksemburg', 'LR': 'Libeeria', 'LS': 'Lesotho', 'TH': 'Tai', 'TF': u'Prantsuse L\xf5unaalad', 'TG': 'Togo', 'TD': 'Tiaad', 'TC': 'Turks ja Caicos', 'LY': u'Liib\xfca', 'VA': u'P\xfcha Tool (Vatikan)', 'VC': 'Saint Vincent ja Grenadiinid', 'AE': u'Araabia \xdchendemiraadid', 'AD': 'Andorra', 'AG': 'Antigua ja Barbuda', 'AF': 'Afganistan', 'AI': 'Anguilla', 'VI': 'USA Neitsisaared', 'IS': 'Island', 'IR': 'Iraan', 'AM': 'Armeenia', 'AL': 'Albaania', 'AO': 'Angola', 'AN': 'Hollandi Antillid', 'AQ': 'Antarktika', 'AS': 'Ameerika Samoa', 'AR': 'Argentina', 'AU': 'Austraalia', 'AT': 'Austria', 'AW': 'Aruba', 'IN': 'India', 'TZ': 'Tansaania', 'AZ': u'Aserbaid\u012aaan', 'IE': 'Iirimaa', 'ID': 'Indoneesia', 'UA': 'Ukraina', 'QA': 'Katar', 'MZ': 'Mosambiik'}

months=['jaanuar', 'veebruar', u'm\xe4rts', 'aprill', 'mai', 'juuni', 'juuli', 'august', 'september', 'oktoober', 'november', 'detsember']

abbrMonths=['jaan', 'veebr', u'm\xe4rts', 'apr', 'mai', 'juuni', 'juuli', 'aug', 'sept', 'okt', 'nov', 'dets']

days=[u'esmasp\xe4ev', u'teisip\xe4ev', u'kolmap\xe4ev', u'neljap\xe4ev', 'reede', u'laup\xe4ev', u'p\xfchap\xe4ev']

abbrDays=['E', 'T', 'K', 'N', 'R', 'L', 'P']

dateFormats={'medium': '%d.%m.%Y', 'full': '%%(dayname)s, %d, %%(monthname)s %Y', 'long': '%d %%(monthname)s %Y', 'short': '%d.%m.%y'}

numericSymbols={'group': u'\xa0', 'nativeZeroDigit': '0', 'exponential': 'E', 'perMille': u'\u2030', 'nan': u'\ufffd', 'decimal': ',', 'percentSign': '%', 'list': ';', 'patternDigit': '#', 'plusSign': '+', 'infinity': u'\u221e', 'minusSign': '-'}
