// Calendar i18n file
// Language: af (Afrikaans)
// Encoding: Any
// Distributed under the same terms as the calendar itself.

Calendar._DN = new Array
("Sondag",
 "Maandag",
 "Dinsdag",
 "Woensdag",
 "Donderdag",
 "Vrydag",
 "Saterdag",
 "Sondag");

Calendar._SDN = new Array
("Son",
 "Maan",
 "Dins",
 "Woen",
 "Dond",
 "Vry",
 "Sat",
 "Son");

Calendar._FD = 0;

Calendar._MN = new Array
("Januarie",
 "Februarie",
 "Maart",
 "April",
 "Mei",
 "Junie",
 "Julie",
 "Augustus",
 "September",
 "Oktober",
 "November",
 "Desember");

Calendar._SMN = new Array
("Jan",
 "Feb",
 "Mrt",
 "Apr",
 "Mei",
 "Jun",
 "Jul",
 "Aug",
 "Sep",
 "Okt",
 "Nov",
 "Des");

// tooltips
Calendar._TT = {};
Calendar._TT["PREV_YEAR"] = "Vorige jaar (hou vir keuselys)";
Calendar._TT["PREV_MONTH"] = "Vorige maand (hou vir keuselys)";
Calendar._TT["GO_TODAY"] = "Gaan na vandag";
Calendar._TT["NEXT_MONTH"] = "Volgende maand (hou vir keuselys)";
Calendar._TT["NEXT_YEAR"] = "Volgende jaar (hou vir keuselys)";
Calendar._TT["SEL_DATE"] = "Kies datum";
Calendar._TT["DRAG_TO_MOVE"] = "Sleep om te skuif";
Calendar._TT["PART_TODAY"] = " (vandag)";
Calendar._TT["DAY_FIRST"] = "Vertoon %s eerste dag van die week";
Calendar._TT["WEEKEND"] = "0,6";
Calendar._TT["CLOSE"] = "Toemaak";
Calendar._TT["TODAY"] = "Vandag";
Calendar._TT["TIME_PART"] = "(Shift-)Click or drag to change value";
Calendar._TT["DEF_DATE_FORMAT"] = "%Y/%m/%d";
Calendar._TT["TT_DATE_FORMAT"] = "%a, %b %e";
Calendar._TT["WK"] = "wk";
Calendar._TT["TIME"] = "Uur:";
